# Copyright (c) 2015 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import mock
import webob.exc

from oslo_config import cfg

from neutron.debug import debug_agent

from neutron.plugins.ml2 import rpc as ml2_rpc

from neutron.tests.common import helpers

from neutron_lib.api.definitions import portbindings
from neutron_lib import constants as const
from neutron_lib import context as n_context
from neutron_lib.plugins import directory

from networking_bgpvpn.neutron.services.service_drivers.bagpipe import bagpipe
from networking_bgpvpn.tests.unit.services import test_plugin


def _expected_formatted_bgpvpn(id, net_id, rt, gateway_mac=None):
        return {'id': id,
                'network_id': net_id,
                'l3vpn': {'import_rt': rt,
                          'export_rt': rt},
                'gateway_mac': gateway_mac}


class TestBagpipeCommon(test_plugin.BgpvpnTestCaseMixin):

    def setUp(self, plugin=None):
        self.mocked_rpc = mock.patch(
            'networking_bagpipe.agent.bgpvpn.rpc_client'
            '.BGPVPNAgentNotifyApi').start().return_value

        self.mock_attach_rpc = self.mocked_rpc.attach_port_on_bgpvpn
        self.mock_detach_rpc = self.mocked_rpc.detach_port_from_bgpvpn
        self.mock_update_rpc = self.mocked_rpc.update_bgpvpn
        self.mock_delete_rpc = self.mocked_rpc.delete_bgpvpn

        provider = ('networking_bgpvpn.neutron.services.service_drivers.'
                    'bagpipe.bagpipe.BaGPipeBGPVPNDriver')
        super(TestBagpipeCommon, self).setUp(service_provider=provider,
                                             core_plugin=plugin)

        self.ctxt = n_context.Context('fake_user', self._tenant_id)

        n_dict = {"name": "netfoo",
                  "tenant_id": self._tenant_id,
                  "admin_state_up": True,
                  "router:external": True,
                  "shared": True}

        self.external_net = {'network':
                             self.plugin.create_network(self.ctxt,
                                                        {'network': n_dict})}


class TestBagpipeServiceDriver(TestBagpipeCommon):

    def test_create_bgpvpn_l2_fails(self):
        bgpvpn_data = copy.copy(self.bgpvpn_data['bgpvpn'])
        bgpvpn_data.update({"type": "l2"})

        # Assert that an error is returned to the client
        bgpvpn_req = self.new_create_request(
            'bgpvpn/bgpvpns', bgpvpn_data)
        res = bgpvpn_req.get_response(self.ext_api)
        self.assertEqual(webob.exc.HTTPBadRequest.code,
                         res.status_int)

    def test_create_bgpvpn_rds_fails(self):
        bgpvpn_data = copy.copy(self.bgpvpn_data)
        bgpvpn_data['bgpvpn'].update({"route_distinguishers": ["4444:55"]})

        # Assert that an error is returned to the client
        bgpvpn_req = self.new_create_request(
            'bgpvpn/bgpvpns', bgpvpn_data)
        res = bgpvpn_req.get_response(self.ext_api)
        self.assertEqual(webob.exc.HTTPBadRequest.code,
                         res.status_int)

    def test_bagpipe_update_bgpvpn_rds_fails(self):
        with self.bgpvpn() as bgpvpn:
            update_data = {'bgpvpn': {"route_distinguishers": ["4444:55"]}}

            self._update('bgpvpn/bgpvpns',
                         bgpvpn['bgpvpn']['id'],
                         update_data,
                         expected_code=webob.exc.HTTPBadRequest.code)
            show_bgpvpn = self._show('bgpvpn/bgpvpns',
                                     bgpvpn['bgpvpn']['id'])
            self.assertEqual([],
                             show_bgpvpn['bgpvpn']['route_distinguishers'])

    def test_bagpipe_associate_net(self):
        with self.port() as port1:
            net_id = port1['port']['network_id']
            with self.bgpvpn() as bgpvpn:
                id = bgpvpn['bgpvpn']['id']
                rt = bgpvpn['bgpvpn']['route_targets']
                self.mock_update_rpc.reset_mock()
                with self.assoc_net(id, net_id):
                    self.mock_update_rpc.assert_called_once_with(
                        mock.ANY,
                        _expected_formatted_bgpvpn(id, net_id, rt))

    def test_bagpipe_associate_external_net_failed(self):
        net_id = self.external_net['network']['id']
        with self.bgpvpn(tenant_id='another_tenant') as bgpvpn:
            id = bgpvpn['bgpvpn']['id']
            data = {'network_association': {'network_id': net_id,
                                            'tenant_id': self._tenant_id}}
            bgpvpn_net_req = self.new_create_request(
                'bgpvpn/bgpvpns',
                data=data,
                fmt=self.fmt,
                id=id,
                subresource='network_associations')
            res = bgpvpn_net_req.get_response(self.ext_api)
            self.assertEqual(res.status_int, webob.exc.HTTPForbidden.code)

    def test_bagpipe_associate_router(self):
        with self.router(tenant_id=self._tenant_id) as router:
            router_id = router['router']['id']
            with self.subnet() as subnet:
                with self.port(subnet=subnet) as port:
                    net_id = port['port']['network_id']
                    subnet_id = subnet['subnet']['id']
                    itf = self._router_interface_action('add', router_id,
                                                        subnet_id, None)

                    itf_port = self.plugin.get_port(self.ctxt, itf['port_id'])

                    with self.bgpvpn() as bgpvpn:
                        id = bgpvpn['bgpvpn']['id']
                        rt = bgpvpn['bgpvpn']['route_targets']
                        self.mock_update_rpc.reset_mock()
                        with self.assoc_router(id, router_id):
                            self.mock_update_rpc.assert_called_once_with(
                                mock.ANY,
                                _expected_formatted_bgpvpn(
                                    id, net_id,
                                    rt,
                                    itf_port['mac_address']))

    def test_bagpipe_disassociate_net(self):
        mocked_delete = self.mocked_rpc.delete_bgpvpn
        with self.port() as port1:
            net_id = port1['port']['network_id']
            with self.bgpvpn() as bgpvpn:
                id = bgpvpn['bgpvpn']['id']
                rt = bgpvpn['bgpvpn']['route_targets']
                with self.assoc_net(id, net_id,
                                    do_disassociate=False) as assoc:
                    mocked_delete.reset_mock()
                    del_req = self.new_delete_request(
                        'bgpvpn/bgpvpns',
                        id,
                        fmt=self.fmt,
                        subresource='network_associations',
                        sub_id=assoc['network_association']['id'])
                    res = del_req.get_response(self.ext_api)
                    if res.status_int >= 400:
                        raise webob.exc.HTTPClientError(code=res.status_int)

                    mocked_delete.assert_called_once_with(
                        mock.ANY,
                        _expected_formatted_bgpvpn(id, net_id, rt))

    def test_bagpipe_update_bgpvpn_rt(self):
        with self.port() as port1:
            net_id = port1['port']['network_id']
            with self.bgpvpn() as bgpvpn:
                id = bgpvpn['bgpvpn']['id']
                rt = ['6543:21']
                with self.assoc_net(id, net_id):
                    update_data = {'bgpvpn': {'route_targets': ['6543:21']}}
                    self.mock_update_rpc.reset_mock()
                    self._update('bgpvpn/bgpvpns',
                                 bgpvpn['bgpvpn']['id'],
                                 update_data)
                    self.mock_update_rpc.assert_called_once_with(
                        mock.ANY,
                        _expected_formatted_bgpvpn(id, net_id, rt))

    def test_bagpipe_delete_bgpvpn(self):
        mocked_delete = self.mocked_rpc.delete_bgpvpn
        with self.port() as port1:
            net_id = port1['port']['network_id']
            with self.bgpvpn(do_delete=False) as bgpvpn:
                id = bgpvpn['bgpvpn']['id']
                rt = bgpvpn['bgpvpn']['route_targets']
                mocked_delete.reset_mock()
                with self.assoc_net(id, net_id, do_disassociate=False):
                    self._delete('bgpvpn/bgpvpns', id)
                    mocked_delete.assert_called_once_with(
                        mock.ANY,
                        _expected_formatted_bgpvpn(id, net_id, rt))

    def test_bagpipe_callback_to_rpc_update_port_after_router_itf_added(self):
        driver = self.bgpvpn_plugin.driver
        with self.network() as net, \
                self.subnet(network=net) as subnet, \
                self.router(tenant_id=self._tenant_id) as router, \
                self.bgpvpn() as bgpvpn:
            itf = self._router_interface_action('add',
                                                router['router']['id'],
                                                subnet['subnet']['id'],
                                                None)
            with self.assoc_router(bgpvpn['bgpvpn']['id'],
                                   router['router']['id']), \
                    self.port(subnet=subnet) as port:
                mac_address = port['port']['mac_address']
                formatted_ip = (port['port']['fixed_ips'][0]['ip_address'] +
                                '/' + subnet['subnet']['cidr'].split('/')[-1])
                itf_port = self.plugin.get_port(self.ctxt, itf['port_id'])

                expected = {
                    'gateway_ip': subnet['subnet']['gateway_ip'],
                    'mac_address': mac_address,
                    'ip_address': formatted_ip,
                    'gateway_mac': itf_port['mac_address']
                }
                expected.update(driver._format_bgpvpn_network_route_targets(
                    [bgpvpn['bgpvpn']]))

                actual = driver._retrieve_bgpvpn_network_info_for_port(
                    self.ctxt, port['port'])

                self.assertEqual(expected, actual)

    def test_bagpipe_get_network_info_for_port(self):
        with self.network() as net, \
                self.subnet(network=net) as subnet, \
                self.router(tenant_id=self._tenant_id) as router, \
                self.port(subnet=subnet) as port:
            itf = self._router_interface_action('add',
                                                router['router']['id'],
                                                subnet['subnet']['id'],
                                                None)
            itf_port = self.plugin.get_port(self.ctxt, itf['port_id'])

            r = bagpipe.get_network_info_for_port(self.ctxt,
                                                  port['port']['id'],
                                                  net['network']['id'])

            expected_ip = port['port']['fixed_ips'][0]['ip_address'] + "/24"
            self.assertEqual({
                'mac_address': port['port']['mac_address'],
                'ip_address': expected_ip,
                'gateway_ip': subnet['subnet']['gateway_ip'],
                'gateway_mac': itf_port['mac_address']
            }, r)

RT = '12345:1'

BGPVPN_INFO = {'mac_address': 'de:ad:00:00:be:ef',
               'ip_address': '10.0.0.2',
               'gateway_ip': '10.0.0.1',
               'l3vpn': {'import_rt': [RT],
                         'export_rt': [RT]
                         },
               'gateway_mac': None
               }


class TestBagpipeServiceDriverCallbacks(TestBagpipeCommon):
    '''Check that receiving callbacks results in RPC calls to the agent'''

    _plugin_name = 'neutron.plugins.ml2.plugin.Ml2Plugin'

    def setUp(self):
        cfg.CONF.set_override('mechanism_drivers',
                              ['logger', 'fake_agent'],
                              'ml2')

        super(TestBagpipeServiceDriverCallbacks, self).setUp(self._plugin_name)

        self.port_create_status = 'DOWN'
        self.plugin = directory.get_plugin()
        self.plugin.start_rpc_listeners()

        self.bagpipe_driver = self.bgpvpn_plugin.driver

        self.patched_driver = mock.patch.object(
            self.bgpvpn_plugin.driver,
            '_retrieve_bgpvpn_network_info_for_port',
            return_value=BGPVPN_INFO)
        self.patched_driver.start()

        # we choose an agent of type const.AGENT_TYPE_OFA
        # because this is the type used by the fake_agent mech driver
        helpers.register_ovs_agent(helpers.HOST, const.AGENT_TYPE_OFA)
        helpers.register_l3_agent()

    def _build_expected_return_active(self, port):
        bgpvpn_info_port = BGPVPN_INFO.copy()
        bgpvpn_info_port.update({'id': port['id'],
                                 'network_id': port['network_id']})
        return bgpvpn_info_port

    def _build_expected_return_down(self, port):
        return {'id': port['id'],
                'network_id': port['network_id']}

    def _update_port_status(self, port, status):
        network_id = port['port']['network_id']
        some_network = {'id': network_id}
        self.plugin.get_network = mock.Mock(return_value=some_network)

        self.plugin.update_port_status(self.ctxt, port['port']['id'],
                                       status, helpers.HOST)

    def test_bagpipe_callback_to_rpc_update_down2active(self):
        with self.port(arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:

            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            self.mock_attach_rpc.assert_called_once_with(
                mock.ANY,
                self._build_expected_return_active(port['port']),
                helpers.HOST)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_update_active2down(self):
        with self.port(arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:

            self._update_port_status(port, const.PORT_STATUS_ACTIVE)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_DOWN)

            self.mock_detach_rpc.assert_called_once_with(
                mock.ANY,
                self._build_expected_return_down(port['port']),
                helpers.HOST)
            self.assertFalse(self.mock_attach_rpc.called)

    def test_bagpipe_callback_to_rpc_update_active2active(self):
        with self.port(arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:

            self._update_port_status(port, const.PORT_STATUS_ACTIVE)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_update_down2down(self):
        with self.port(arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:

            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_DOWN)

            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_deleted(self):
        with self.port(arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()

            self.plugin.delete_port(self.ctxt, port['port']['id'])

            self.mock_detach_rpc.assert_called_once_with(
                mock.ANY,
                self._build_expected_return_down(port['port']),
                helpers.HOST)
            self.assertFalse(self.mock_attach_rpc.called)

    def test_bagpipe_callback_to_rpc_update_active_ignore_net_ports(self):
        with self.port(device_owner=const.DEVICE_OWNER_NETWORK_PREFIX,
                       arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_dont_ignore_probe_ports_compute(self):
        with self.port(device_owner=debug_agent.DEVICE_OWNER_COMPUTE_PROBE,
                       arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            self.mock_attach_rpc.assert_called_once_with(
                mock.ANY,
                self._build_expected_return_active(port['port']),
                helpers.HOST)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_dont_ignore_probe_ports_network(self):
        with self.port(device_owner=debug_agent.DEVICE_OWNER_NETWORK_PROBE,
                       arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            self.mock_attach_rpc.assert_called_once_with(
                mock.ANY,
                self._build_expected_return_active(port['port']),
                helpers.HOST)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_update_down_ignore_net_ports(self):
        with self.port(device_owner=const.DEVICE_OWNER_NETWORK_PREFIX,
                       arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_deleted_ignore_net_ports(self):
        with self.port(device_owner=const.DEVICE_OWNER_NETWORK_PREFIX,
                       arg_list=(portbindings.HOST_ID,),
                       **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()

            self.bagpipe_driver.registry_port_deleted(
                None, None, None,
                context=self.ctxt,
                port_id=port['port']['id']
            )
            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_update_active_ignore_external_net(self):
        with self.subnet(network=self.external_net) as subnet, \
                self.port(subnet=subnet,
                          arg_list=(portbindings.HOST_ID,),
                          **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_update_down_ignore_external_net(self):
        with self.subnet(network=self.external_net) as subnet, \
                self.port(subnet=subnet,
                          arg_list=(portbindings.HOST_ID,),
                          **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()
            self._update_port_status(port, const.PORT_STATUS_DOWN)

            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_bagpipe_callback_to_rpc_deleted_ignore_external_net(self):
        with self.subnet(network=self.external_net) as subnet, \
                self.port(subnet=subnet,
                          arg_list=(portbindings.HOST_ID,),
                          **{portbindings.HOST_ID: helpers.HOST}) as port:
            self._update_port_status(port, const.PORT_STATUS_DOWN)
            self.mock_attach_rpc.reset_mock()
            self.mock_detach_rpc.reset_mock()

            self.bagpipe_driver.registry_port_deleted(
                None, None, None,
                context=self.ctxt,
                port_id=port['port']['id']
            )
            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)

    def test_delete_port_to_bgpvpn_rpc(self):
        with self.network() as net, \
            self.subnet(network=net) as subnet, \
            self.port(subnet=subnet,
                      arg_list=(portbindings.HOST_ID,),
                      **{portbindings.HOST_ID: helpers.HOST}) as port, \
            mock.patch.object(self.plugin, 'get_port',
                              return_value=port['port']), \
            mock.patch.object(self.plugin, 'get_network',
                              return_value=net['network']):

            self.plugin.delete_port(self.ctxt, port['port']['id'])

            self.mock_detach_rpc.assert_called_once_with(
                mock.ANY,
                self._build_expected_return_down(port['port']),
                helpers.HOST)

    def test_bagpipe_callback_to_rpc_update_port_router_itf_added(self):
        with self.network() as net, \
                self.subnet(network=net) as subnet, \
                self.port(subnet=subnet) as port, \
                self.router(tenant_id=self._tenant_id) as router, \
                self.bgpvpn() as bgpvpn, \
                mock.patch.object(self.bagpipe_driver, 'get_bgpvpn',
                                  return_value=bgpvpn['bgpvpn']),\
                mock.patch.object(bagpipe,
                                  'get_router_bgpvpn_assocs',
                                  return_value=[{
                                      'bgpvpn_id': bgpvpn['bgpvpn']['id']
                                  }]).start():
            self.bagpipe_driver.registry_router_interface_created(
                None, None, None,
                context=self.ctxt,
                port={'network_id': net['network']['id']},
                router_id=router['router']['id'],
            )
            self.mock_update_rpc.assert_called_once_with(
                mock.ANY,
                self.bagpipe_driver._format_bgpvpn(self.ctxt,
                                                   bgpvpn['bgpvpn'],
                                                   port['port']['network_id']))

    def test_bagpipe_callback_to_rpc_update_port_router_itf_removed(self):
        with self.network() as net, \
                self.subnet(network=net) as subnet, \
                self.port(subnet=subnet) as port, \
                self.router(tenant_id=self._tenant_id) as router, \
                self.bgpvpn() as bgpvpn, \
                mock.patch.object(self.bagpipe_driver, 'get_bgpvpn',
                                  return_value=bgpvpn['bgpvpn']),\
                mock.patch.object(bagpipe,
                                  'get_router_bgpvpn_assocs',
                                  return_value=[{
                                      'bgpvpn_id': bgpvpn['bgpvpn']['id']
                                  }]).start():
            self.bagpipe_driver.registry_router_interface_deleted(
                None, None, None,
                context=self.ctxt,
                network_id=port['port']['network_id'],
                port={'device_id': router['router']['id'],
                      'network_id': net['network']['id']}
            )
            self.mock_delete_rpc.assert_called_once_with(
                mock.ANY,
                self.bagpipe_driver._format_bgpvpn(self.ctxt,
                                                   bgpvpn['bgpvpn'],
                                                   port['port']['network_id']))

    def test_l3agent_add_remove_router_interface_to_bgpvpn_rpc(self):
        with self.network() as net, \
                self.subnet(network=net) as subnet, \
                self.router(tenant_id=self._tenant_id) as router, \
                self.bgpvpn() as bgpvpn, \
                self.port(subnet=subnet,
                          arg_list=(portbindings.HOST_ID,),
                          **{portbindings.HOST_ID: helpers.HOST}), \
                mock.patch.object(bagpipe,
                                  'get_router_bgpvpn_assocs',
                                  return_value=[{
                                      'bgpvpn_id': bgpvpn['bgpvpn']['id']
                                  }]).start():
            self._router_interface_action('add',
                                          router['router']['id'],
                                          subnet['subnet']['id'],
                                          None)
            self.mock_update_rpc.assert_called_once_with(
                mock.ANY,
                self.bagpipe_driver._format_bgpvpn(self.ctxt,
                                                   bgpvpn['bgpvpn'],
                                                   net['network']['id']))
            self._router_interface_action('remove',
                                          router['router']['id'],
                                          subnet['subnet']['id'],
                                          None)
            self.mock_delete_rpc.assert_called_once_with(
                mock.ANY,
                self.bagpipe_driver._format_bgpvpn(self.ctxt,
                                                   bgpvpn['bgpvpn'],
                                                   net['network']['id']))

    def test_gateway_mac_info_rpc(self):
        BGPVPN_INFO_GW_MAC = copy.copy(BGPVPN_INFO)
        BGPVPN_INFO_GW_MAC.update(gateway_mac='aa:bb:cc:dd:ee:ff')
        self.patched_driver.stop()
        with self.network() as net, \
                self.subnet(network=net) as subnet, \
                self.router(tenant_id=self._tenant_id) as router, \
                self.bgpvpn(route_targets=[RT]) as bgpvpn, \
                self.port(subnet=subnet,
                          arg_list=(portbindings.HOST_ID,),
                          **{portbindings.HOST_ID: helpers.HOST}) as port, \
                self.assoc_net(bgpvpn['bgpvpn']['id'],
                               net['network']['id']), \
                mock.patch.object(self.bgpvpn_plugin.driver,
                                  'retrieve_bgpvpns_of_router_assocs'
                                  '_by_network',
                                  return_value=[{'type': 'l3',
                                                 'route_targets': [RT]}]
                                  ):
            self._update_port_status(port, const.PORT_STATUS_ACTIVE)

            itf = self._router_interface_action('add',
                                                router['router']['id'],
                                                subnet['subnet']['id'],
                                                None)
            itf_port = self.plugin.get_port(self.ctxt, itf['port_id'])

            self.mock_update_rpc.assert_called_with(
                mock.ANY,
                _expected_formatted_bgpvpn(bgpvpn['bgpvpn']['id'],
                                           net['network']['id'],
                                           [RT],
                                           gateway_mac=itf_port['mac_address'])
            )

            self._router_interface_action('remove',
                                          router['router']['id'],
                                          subnet['subnet']['id'],
                                          None)

            self.mock_update_rpc.assert_called_with(
                mock.ANY,
                _expected_formatted_bgpvpn(bgpvpn['bgpvpn']['id'],
                                           net['network']['id'],
                                           [RT],
                                           gateway_mac=None)
            )

        self.patched_driver.start()

    def test_l2agent_rpc_to_bgpvpn_rpc(self):
        #
        # Test that really simulate the ML2 codepath that
        # generate the registry events.

        ml2_rpc_callbacks = ml2_rpc.RpcCallbacks(mock.Mock(), mock.Mock())

        n_dict = {"name": "netfoo",
                  "tenant_id": self._tenant_id,
                  "admin_state_up": True,
                  "shared": False}

        net = self.plugin.create_network(self.ctxt, {'network': n_dict})

        subnet_dict = {'name': 'test_subnet',
                       'tenant_id': self._tenant_id,
                       'ip_version': 4,
                       'cidr': '10.0.0.0/24',
                       'allocation_pools': [{'start': '10.0.0.2',
                                             'end': '10.0.0.254'}],
                       'enable_dhcp': False,
                       'dns_nameservers': [],
                       'host_routes': [],
                       'network_id': net['id']}

        self.plugin.create_subnet(self.ctxt, {'subnet': subnet_dict})

        p_dict = {'network_id': net['id'],
                  'tenant_id': self._tenant_id,
                  'name': 'fooport',
                  "admin_state_up": True,
                  "device_id": "tapfoo",
                  "device_owner": "not_me",
                  "mac_address": "de:ad:00:00:be:ef",
                  "fixed_ips": [],
                  "binding:host_id": helpers.HOST,
                  }

        port = self.plugin.create_port(self.ctxt, {'port': p_dict})

        ml2_rpc_callbacks.update_device_up(self.ctxt,
                                           host=helpers.HOST,
                                           agent_id='fooagent',
                                           device="de:ad:00:00:be:ef")
        self.mock_attach_rpc.assert_called_once_with(
            mock.ANY,
            self._build_expected_return_active(port),
            helpers.HOST)

        ml2_rpc_callbacks.update_device_down(self.ctxt,
                                             host=helpers.HOST,
                                             agent_id='fooagent',
                                             device="de:ad:00:00:be:ef")

        self.mock_detach_rpc.assert_called_once_with(
            mock.ANY,
            self._build_expected_return_down(port),
            helpers.HOST)
        self.mock_detach_rpc.reset_mock()

        self.plugin.delete_port(self.ctxt, port['id'])

        self.mock_detach_rpc.assert_called_once_with(
            mock.ANY,
            self._build_expected_return_down(port),
            helpers.HOST)

    def test_exception_on_callback(self):
        with mock.patch.object(bagpipe.LOG, 'exception') as log_exc:
            self.bagpipe_driver.registry_port_updated(
                None, None, None,
                context=self.ctxt,
                port=None
            )
            self.assertFalse(self.mock_attach_rpc.called)
            self.assertFalse(self.mock_detach_rpc.called)
            self.assertTrue(log_exc.called)



#
# Copyright (C) 2015 Ericsson India Global Services Pvt Ltd.
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
#

import requests

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import excutils

from networking_odl.common import client as odl_client

from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
from networking_bgpvpn.neutron.services.service_drivers import driver_api

cfg.CONF.import_group('ml2_odl', 'networking_odl.common.config')

LOG = logging.getLogger(__name__)
BGPVPNS = 'bgpvpns'
OPENDAYLIGHT_BGPVPN_DRIVER_NAME = 'OpenDaylight'


class OpenDaylightBgpvpnDriver(driver_api.BGPVPNDriver):

    """OpenDaylight BGPVPN Driver

    This code is the backend implementation for the OpenDaylight BGPVPN
    driver for Openstack Neutron.
    """

    def __init__(self, service_plugin):
        LOG.warning("This OpenDaylight BGPVPN driver has been deprecated"
                    "and will be removed. Switch to new v2 Driver: "
                    "BGPVPN:OpenDaylight:networking_odl.bgpvpn.odl_v2."
                    "OpenDaylightBgpvpnDriver:default")
        super(OpenDaylightBgpvpnDriver, self).__init__(service_plugin)
        self.service_plugin = service_plugin

        self.client = odl_client.OpenDaylightRestClient.create_client()

    def create_bgpvpn_precommit(self, context, bgpvpn):
        pass

    def create_bgpvpn_postcommit(self, context, bgpvpn):
        url = BGPVPNS
        try:
            self.client.sendjson('post', url, {BGPVPNS[:-1]: bgpvpn})
        except requests.exceptions.RequestException:
            with excutils.save_and_reraise_exception():
                # delete from db
                d_bgpvpn = self.bgpvpn_db.delete_bgpvpn(context, bgpvpn['id'])
                LOG.debug("Deleted bgpvpn %s from db", d_bgpvpn)

    def delete_bgpvpn_postcommit(self, context, bgpvpn):
        url = BGPVPNS + '/' + bgpvpn['id']
        self.client.sendjson('delete', url, None)

    def update_bgpvpn_postcommit(self, context, old_bgpvpn, bgpvpn):
        url = BGPVPNS + '/' + bgpvpn['id']
        self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})

    def create_net_assoc_precommit(self, context, net_assoc):
        bgpvpns = self.bgpvpn_db.get_bgpvpns(
            context,
            filters={
                'networks': [net_assoc['network_id']],
            },
        )
        if len(bgpvpns) > 1:
            raise bgpvpn_ext.BGPVPNNetworkAssocExistsAnotherBgpvpn(
                driver=OPENDAYLIGHT_BGPVPN_DRIVER_NAME,
                network=net_assoc['network_id'],
                bgpvpn=bgpvpns[0]['id'])

    def create_net_assoc_postcommit(self, context, net_assoc):
        bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
        url = BGPVPNS + '/' + bgpvpn['id']
        try:
            self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})
        except requests.exceptions.RequestException:
            with excutils.save_and_reraise_exception():
                # delete from db
                d_netassoc = self.bgpvpn_db.delete_net_assoc(
                    context, net_assoc['id'], net_assoc['bgpvpn_id'])
                LOG.debug("Deleted net_assoc %s from db", d_netassoc)

    def delete_net_assoc_postcommit(self, context, net_assoc):
        bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
        url = BGPVPNS + '/' + bgpvpn['id']
        self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})

    def create_router_assoc_precommit(self, context, router_assoc):
        associated_routers = self.get_router_assocs(context,
                                                    router_assoc['bgpvpn_id'])
        for assoc_router in associated_routers:
            if(router_assoc["router_id"] != assoc_router["router_id"]):
                raise bgpvpn_ext.BGPVPNMultipleRouterAssocNotSupported(
                    driver=OPENDAYLIGHT_BGPVPN_DRIVER_NAME)

    def create_router_assoc_postcommit(self, context, router_assoc):
        bgpvpn = self.get_bgpvpn(context, router_assoc['bgpvpn_id'])
        url = BGPVPNS + '/' + bgpvpn['id']
        try:
            self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})
        except requests.exceptions.RequestException:
            with excutils.save_and_reraise_exception():
                # delete from db
                d_routerassoc = self.bgpvpn_db.delete_router_assoc(
                    context, router_assoc['id'], router_assoc['bgpvpn_id'])
                LOG.debug("Deleted router_assoc %s from db", d_routerassoc)

    def delete_router_assoc_postcommit(self, context, router_assoc):
        bgpvpn = self.get_bgpvpn(context, router_assoc['bgpvpn_id'])
        url = BGPVPNS + '/' + bgpvpn['id']
        self.client.sendjson('put', url, {BGPVPNS[:-1]: bgpvpn})



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import collections
import logging

from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api
from bgpvpn_dashboard.common import bgpvpn as bgpvpn_common

LOG = logging.getLogger(__name__)


class CommonData(forms.SelfHandlingForm):
    fields_order = []
    name = forms.CharField(max_length=255,
                           label=_("Name"),
                           required=False)

    failure_url = reverse_lazy('horizon:project:bgpvpn:index')

    def __init__(self, request, *args, **kwargs):
        super(CommonData, self).__init__(request, *args, **kwargs)
        if 'keyOrder' in self.fields:
            self.fields.keyOrder = self.fields_order
        else:
            self.fields = collections.OrderedDict(
                (k, self.fields[k]) for k in self.fields_order)

    def clean(self):
        cleaned_data = super(CommonData, self).clean()
        name = cleaned_data.get('name')
        try:
            if self.request.user.is_superuser:
                for attribute in bgpvpn_common.RT_FORMAT_ATTRIBUTES:
                    if not cleaned_data.get(attribute):
                        del cleaned_data[attribute]
                    else:
                        cleaned_data[attribute] = bgpvpn_common.format_rt(
                            cleaned_data[attribute])
                # if an admin user use the bgpvpn panel project
                # tenant_id field doesn't exist
                if not cleaned_data.get('tenant_id'):
                    tenant_id = self.request.user.tenant_id
                else:
                    tenant_id = cleaned_data.get('tenant_id')
                bgpvpn_api.bgpvpns_list(
                    self.request, name=name, tenant_id=tenant_id)
            else:
                bgpvpn_api.bgpvpns_list(self.request, name=name)
        except Exception:
            msg = _('Unable to get BGPVPN with name %s') % name
            exceptions.check_message(["Connection", "refused"], msg)
            raise
        return cleaned_data

    @staticmethod
    def _del_attributes(attributes, data):
        for attribute in attributes:
            del data[attribute]

    def handle(self, request, data):
        params = {}
        params.update(data)
        try:
            if self.action == 'update':
                if request.user.is_superuser and data.get('tenant_id'):
                    attributes = ('bgpvpn_id', 'type', 'tenant_id')
                else:
                    attributes = ('bgpvpn_id', 'type')
                self._del_attributes(attributes, params)
                bgpvpn = bgpvpn_api.bgpvpn_update(request,
                                                  data['bgpvpn_id'],
                                                  **params)
                success_action = 'modified'
            elif self.action == 'create':
                success_action = 'created'
                bgpvpn = bgpvpn_api.bgpvpn_create(request, **params)
            else:
                raise Exception(
                    _('Action type %s is not supported') % self.action)
            msg = _('BGPVPN {name} was successfully {action}.').format(
                name=data['name'],
                action=success_action)
            LOG.debug(msg)
            messages.success(request, msg)
            return bgpvpn
        except Exception:
            msg = _('Failed to {action} BGPVPN {name}').format(
                action=self.action,
                name=data['name'])
            exceptions.handle(request, msg, redirect=self.failure_url)
            return False


class EditDataBgpVpn(CommonData):
    bgpvpn_id = forms.CharField(label=_("ID"), widget=forms.TextInput(
        attrs={'readonly': 'readonly'}))
    type = forms.CharField(label=_("Type"), widget=forms.TextInput(
        attrs={'readonly': 'readonly'}))
    fields_order = ['name', 'bgpvpn_id', 'type']

    def __init__(self, request, *args, **kwargs):
        super(EditDataBgpVpn, self).__init__(request, *args, **kwargs)
        self.action = 'update'



# Copyright (c) 2015 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron.db import servicetype_db as st_db
from neutron.services import provider_configuration as pconf
from neutron.services import service_base

from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
from neutron_lib.callbacks import events
from neutron_lib.callbacks import registry
from neutron_lib.callbacks import resources
from neutron_lib import constants as const
from neutron_lib import exceptions as n_exc
from neutron_lib.plugins import constants as plugin_constants
from neutron_lib.plugins import directory

from oslo_log import log

from networking_bgpvpn._i18n import _

from networking_bgpvpn.neutron.extensions import bgpvpn
from networking_bgpvpn.neutron.services.common import constants

LOG = log.getLogger(__name__)


class BGPVPNPlugin(bgpvpn.BGPVPNPluginBase):
    supported_extension_aliases = ["bgpvpn"]
    path_prefix = "/bgpvpn"

    def __init__(self):
        super(BGPVPNPlugin, self).__init__()

        # Need to look into /etc/neutron/networking_bgpvpn.conf for
        # service_provider definitions:
        service_type_manager = st_db.ServiceTypeManager.get_instance()
        service_type_manager.add_provider_configuration(
            bgpvpn_def.LABEL,
            pconf.ProviderConfiguration('networking_bgpvpn'))

        # Load the default driver
        drivers, default_provider = service_base.load_drivers(bgpvpn_def.LABEL,
                                                              self)
        LOG.info("BGP VPN Service Plugin using Service Driver: %s",
                 default_provider)
        self.driver = drivers[default_provider]

        if len(drivers) > 1:
            LOG.warning("Multiple drivers configured for BGPVPN, although"
                        "running multiple drivers in parallel is not yet"
                        "supported")
        registry.subscribe(self._notify_adding_interface_to_router,
                           resources.ROUTER_INTERFACE,
                           events.BEFORE_CREATE)

    def _notify_adding_interface_to_router(self, resource, event, trigger,
                                           **kwargs):
        context = kwargs.get('context')
        network_id = kwargs.get('network_id')
        router_id = kwargs.get('router_id')
        try:
            routers_bgpvpns = self.driver.get_bgpvpns(
                context,
                filters={
                    'routers': [router_id],
                },
            )
        except bgpvpn.BGPVPNRouterAssociationNotSupported:
            return
        nets_bgpvpns = self.driver.get_bgpvpns(
            context,
            filters={
                'networks': [network_id],
                'type': [constants.BGPVPN_L3],
            },
        )

        if routers_bgpvpns and nets_bgpvpns:
            msg = _('It is not allowed to add an interface to a router if '
                    'both the router and the network are bound to an '
                    'L3 BGPVPN.')
            raise n_exc.BadRequest(resource='bgpvpn', msg=msg)

    def _validate_network(self, context, net_id):
        plugin = directory.get_plugin()
        network = plugin.get_network(context, net_id)
        self._validate_network_has_router_assoc(context, network, plugin)
        return network

    def _validate_network_has_router_assoc(self, context, network, plugin):
        filter = {'network_id': [network['id']],
                  'device_owner': [const.DEVICE_OWNER_ROUTER_INTF]}
        router_port = plugin.get_ports(context, filters=filter)
        if router_port:
            router_id = router_port[0]['device_id']
            filter = {'tenant_id': [network['tenant_id']]}
            bgpvpns = self.driver.get_bgpvpns(context, filters=filter)
            bgpvpns = [str(bgpvpn['id']) for bgpvpn in bgpvpns
                       if router_id in bgpvpn['routers']]
            if bgpvpns:
                msg = ('Network %(net_id)s is linked to a router which is '
                       'already associated to bgpvpn(s) %(bgpvpns)s'
                       % {'net_id': network['id'],
                          'bgpvpns': bgpvpns}
                       )
                raise n_exc.BadRequest(resource='bgpvpn', msg=msg)

    def _validate_router(self, context, router_id):
        l3_plugin = directory.get_plugin(plugin_constants.L3)
        router = l3_plugin.get_router(context, router_id)
        plugin = directory.get_plugin()
        self._validate_router_has_net_assocs(context, router, plugin)
        return router

    def _validate_router_has_net_assocs(self, context, router, plugin):
        filter = {'device_id': [router['id']],
                  'device_owner': [const.DEVICE_OWNER_ROUTER_INTF]}
        router_ports = plugin.get_ports(context, filters=filter)
        if router_ports:
            filter = {'tenant_id': [router['tenant_id']]}
            bgpvpns = self.driver.get_bgpvpns(context, filters=filter)
            for port in router_ports:
                bgpvpns = [str(bgpvpn['id']) for bgpvpn in bgpvpns
                           if port['network_id'] in bgpvpn['networks']]
                if bgpvpns:
                    msg = ('router %(rtr_id)s has an attached network '
                           '%(net_id)s which is already associated to '
                           'bgpvpn(s) %(bgpvpns)s'
                           % {'rtr_id': router['id'],
                              'net_id': port['network_id'],
                              'bgpvpns': bgpvpns})
                    raise n_exc.BadRequest(resource='bgpvpn', msg=msg)

    def get_plugin_type(self):
        return bgpvpn_def.LABEL

    def get_plugin_description(self):
        return "Neutron BGPVPN Service Plugin"

    def create_bgpvpn(self, context, bgpvpn):
        bgpvpn = bgpvpn['bgpvpn']
        return self.driver.create_bgpvpn(context, bgpvpn)

    def get_bgpvpns(self, context, filters=None, fields=None):
        return self.driver.get_bgpvpns(context, filters, fields)

    def get_bgpvpn(self, context, id, fields=None):
        return self.driver.get_bgpvpn(context, id, fields)

    def update_bgpvpn(self, context, id, bgpvpn):
        bgpvpn = bgpvpn['bgpvpn']
        return self.driver.update_bgpvpn(context, id, bgpvpn)

    def delete_bgpvpn(self, context, id):
        self.driver.delete_bgpvpn(context, id)

    def create_bgpvpn_network_association(self, context, bgpvpn_id,
                                          network_association):
        net_assoc = network_association['network_association']
        # check net exists
        net = self._validate_network(context, net_assoc['network_id'])
        # check every resource belong to the same tenant
        bgpvpn = self.get_bgpvpn(context, bgpvpn_id)
        if net['tenant_id'] != bgpvpn['tenant_id']:
            msg = 'network doesn\'t belong to the bgpvpn owner'
            raise n_exc.NotAuthorized(resource='bgpvpn', msg=msg)
        if net_assoc['tenant_id'] != bgpvpn['tenant_id']:
            msg = 'network association and bgpvpn should belong to\
                the same tenant'
            raise n_exc.NotAuthorized(resource='bgpvpn', msg=msg)
        return self.driver.create_net_assoc(context, bgpvpn_id, net_assoc)

    def get_bgpvpn_network_association(self, context, assoc_id, bgpvpn_id,
                                       fields=None):
        return self.driver.get_net_assoc(context, assoc_id, bgpvpn_id, fields)

    def get_bgpvpn_network_associations(self, context, bgpvpn_id,
                                        filters=None, fields=None):
        return self.driver.get_net_assocs(context, bgpvpn_id, filters, fields)

    def update_bgpvpn_network_association(self, context, assoc_id, bgpvpn_id,
                                          network_association):
        # TODO(matrohon) : raise an unsuppported error
        pass

    def delete_bgpvpn_network_association(self, context, assoc_id, bgpvpn_id):
        self.driver.delete_net_assoc(context, assoc_id, bgpvpn_id)

    def create_bgpvpn_router_association(self, context, bgpvpn_id,
                                         router_association):
        router_assoc = router_association['router_association']
        router = self._validate_router(context, router_assoc['router_id'])
        bgpvpn = self.get_bgpvpn(context, bgpvpn_id)
        if not bgpvpn['type'] == constants.BGPVPN_L3:
            msg = ("Router associations require the bgpvpn to be of type %s"
                   % constants.BGPVPN_L3)
            raise n_exc.BadRequest(resource='bgpvpn', msg=msg)
        if not router['tenant_id'] == bgpvpn['tenant_id']:
            msg = "router doesn't belong to the bgpvpn owner"
            raise n_exc.NotAuthorized(resource='bgpvpn', msg=msg)
        if not (router_assoc['tenant_id'] == bgpvpn['tenant_id']):
            msg = "router association and bgpvpn should " \
                  "belong to the same tenant"
            raise n_exc.NotAuthorized(resource='bgpvpn', msg=msg)
        return self.driver.create_router_assoc(context, bgpvpn_id,
                                               router_assoc)

    def get_bgpvpn_router_association(self, context, assoc_id, bgpvpn_id,
                                      fields=None):
        return self.driver.get_router_assoc(context, assoc_id, bgpvpn_id,
                                            fields)

    def get_bgpvpn_router_associations(self, context, bgpvpn_id, filters=None,
                                       fields=None):
        return self.driver.get_router_assocs(context, bgpvpn_id, filters,
                                             fields)

    def delete_bgpvpn_router_association(self, context, assoc_id, bgpvpn_id):
        self.driver.delete_router_assoc(context, assoc_id, bgpvpn_id)



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
from django.utils import html
from django.utils.http import urlencode
from django.utils import safestring
from django.utils.translation import ugettext_lazy as _
from horizon import tables


class EditInfoBgpVpn(tables.LinkAction):
    name = "update_info"
    verbose_name = _("Edit BGPVPN")
    url = "horizon:project:bgpvpn:edit"
    classes = ("ajax-modal",)
    icon = "pencil"


class UpdateNetworkAssociations(tables.LinkAction):
    name = "update_network_associations"
    verbose_name = _("Update Network Associations")
    url = "horizon:project:bgpvpn:update-associations"
    classes = ("ajax-modal",)
    icon = "pencil"

    def get_link_url(self, bgpvpn):
        step = 'update_bgpvpn_network'
        base_url = reverse(self.url, args=[bgpvpn.id])
        param = urlencode({"step": step})
        return "?".join([base_url, param])


class UpdateRouterAssociations(tables.LinkAction):
    name = "update_router_associations"
    verbose_name = _("Update Router Associations")
    url = "horizon:project:bgpvpn:update-associations"
    classes = ("ajax-modal",)
    icon = "pencil"

    def get_link_url(self, bgpvpn):
        step = 'update_bgpvpn_router'
        base_url = reverse(self.url, args=[bgpvpn.id])
        param = urlencode({"step": step})
        return "?".join([base_url, param])


def get_network_url(network):
    url = reverse('horizon:project:networks:detail', args=[network.id])
    instance = '<a href=%s>%s</a>' % (url, html.escape(network.name_or_id))
    return instance


def get_router_url(router):
    url = reverse('horizon:project:routers:detail', args=[router.id])
    instance = '<a href=%s>%s</a>' % (url, html.escape(router.name_or_id))
    return instance


class NetworksColumn(tables.Column):
    def get_raw_data(self, bgpvpn):
        networks = [get_network_url(network) for network in bgpvpn.networks]
        return safestring.mark_safe(', '.join(networks))


class RoutersColumn(tables.Column):
    def get_raw_data(self, bgpvpn):
        routers = [get_router_url(router) for router in bgpvpn.routers]
        return safestring.mark_safe(', '.join(routers))


class BgpvpnTable(tables.DataTable):
    name = tables.Column("name_or_id",
                         verbose_name=_("Name"),
                         link=("horizon:project:bgpvpn:detail"))
    type = tables.Column("type", verbose_name=_("Type"))
    networks = NetworksColumn("networks", verbose_name=_("Networks"))
    routers = RoutersColumn("routers", verbose_name=_("Routers"))

    class Meta(object):
        name = "bgpvpns"
        verbose_name = _("BGPVPN")
        row_actions = (EditInfoBgpVpn,
                       UpdateNetworkAssociations,
                       UpdateRouterAssociations)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Default to Horizons test settings to avoid any missing keys
from horizon.test.settings import *  # noqa
from openstack_dashboard.test.settings import *  # noqa

# pop these keys to avoid log warnings about deprecation
# update_dashboards will populate them anyway
HORIZON_CONFIG.pop('dashboards', None)
HORIZON_CONFIG.pop('default_dashboard', None)

import bgpvpn_dashboard.enabled
import openstack_dashboard.enabled

from openstack_dashboard.utils import settings

settings.update_dashboards(
    [
        bgpvpn_dashboard.enabled,
        openstack_dashboard.enabled,
    ],
    HORIZON_CONFIG,
    INSTALLED_APPS
)

# Ensure any duplicate apps are removed after the update_dashboards call
INSTALLED_APPS = list(set(INSTALLED_APPS))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common import exception
from heat.engine import attributes
from heat.engine.clients.os.neutron import neutron_constraints
from heat.engine import constraints
from heat.engine import properties
from heat.engine.resources.openstack.neutron import neutron

from networking_bgpvpn_heat._i18n import _


class BGPVPN(neutron.NeutronResource):
    """A resource for BGPVPN service in neutron.

    """

    PROPERTIES = (
        NAME, TYPE, DESCRIPTION, ROUTE_DISTINGUISHERS,
        IMPORT_TARGETS, EXPORT_TARGETS, ROUTE_TARGETS,
        TENANT_ID
    ) = (
        'name', 'type', 'description',
        'route_distinguishers', 'import_targets',
        'export_targets', 'route_targets',
        'tenant_id'
    )

    ATTRIBUTES = (
        SHOW, STATUS
    ) = (
        'show', 'status'
    )

    properties_schema = {
        NAME: properties.Schema(
            properties.Schema.STRING,
            _('Name for the bgpvpn.'),
        ),
        TYPE: properties.Schema(
            properties.Schema.STRING,
            _('BGP VPN type selection between L3VPN (l3) and '
              'EVPN (l2), default:l3'),
            required=False,
            default='l3',
            constraints=[
                constraints.AllowedValues(['l2', 'l3'])
            ]
        ),
        DESCRIPTION: properties.Schema(
            properties.Schema.STRING,
            _('Description for the bgpvpn.'),
            required=False,
        ),
        TENANT_ID: properties.Schema(
            properties.Schema.STRING,
            _('Tenant this bgpvpn belongs to (name or id).'),
            required=False,
            constraints=[
                constraints.CustomConstraint('keystone.project')
            ]
        ),
        ROUTE_DISTINGUISHERS: properties.Schema(
            properties.Schema.LIST,
            _('List of RDs that will be used to advertize BGPVPN routes.'),
            required=False,
            # TODO(tmorin): add a pattern constraint
        ),
        IMPORT_TARGETS: properties.Schema(
            properties.Schema.LIST,
            _('List of additional Route Targets to import from.'),
            required=False,
            # TODO(tmorin): add a pattern constraint
        ),
        EXPORT_TARGETS: properties.Schema(
            properties.Schema.LIST,
            _('List of additional Route Targets to export to.'),
            required=False,
            # TODO(tmorin): add a pattern constraint
        ),
        ROUTE_TARGETS: properties.Schema(
            properties.Schema.LIST,
            _('Route Targets list to import/export for this BGPVPN.'),
            required=False,
            # TODO(tmorin): add a pattern constraint
        ),
    }

    attributes_schema = {
        STATUS: attributes.Schema(
            _('Status of bgpvpn.'),
        ),
        SHOW: attributes.Schema(
            _('All attributes.')
        ),
    }

    def validate(self):
        super(BGPVPN, self).validate()

    def handle_create(self):
        props = self.prepare_properties(
            self.properties,
            self.physical_resource_name())

        if 'tenant_id' in props:
            tenant_id = self.client_plugin('keystone').get_project_id(
                props['tenant_id'])
            props['tenant_id'] = tenant_id

        bgpvpn = self.neutron().create_bgpvpn({'bgpvpn': props})
        self.resource_id_set(bgpvpn['bgpvpn']['id'])

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        raise NotImplementedError()

    def handle_delete(self):
        try:
            self.neutron().delete_bgpvpn(self.resource_id)
        except Exception as ex:
            self.client_plugin().ignore_not_found(ex)
        else:
            return True

    def _confirm_delete(self):
        while True:
            try:
                yield self._show_resource()
            except exception.NotFound:
                return

    def _show_resource(self):
        return self.neutron().show_bgpvpn(self.resource_id)


# this class is registered to Heat via a setuptools entry point
# (see setup.cfg)
class BGPVPNConstraint(neutron_constraints.NeutronConstraint):
    resource_name = 'bgpvpn'


class BGPVPNNetAssoc(neutron.NeutronResource):

    """A resource for BGPVPNNetAssoc in neutron.

    """

    PROPERTIES = (
        BGPVPN_ID, NETWORK_ID
    ) = (
        'bgpvpn_id', 'network_id'
    )

    ATTRIBUTES = (
        SHOW, STATUS
    ) = (
        'show', 'status'
    )

    properties_schema = {
        BGPVPN_ID: properties.Schema(
            properties.Schema.STRING,
            _('name or ID of the BGPVPN.'),
            required=True,
            constraints=[
                constraints.CustomConstraint('neutron.bgpvpn')
            ]
        ),
        NETWORK_ID: properties.Schema(
            properties.Schema.STRING,
            _('Network which shall be associated with the BGPVPN.'),
            required=True,
            constraints=[
                constraints.CustomConstraint('neutron.network')
            ]
        )
    }

    attributes_schema = {
        STATUS: attributes.Schema(
            _('Status of bgpvpn.'),
        ),
        SHOW: attributes.Schema(
            _('All attributes.')
        ),
    }

    def validate(self):
        super(BGPVPNNetAssoc, self).validate()

    def handle_create(self):
        self.props = self.prepare_properties(self.properties,
                                             self.physical_resource_name())

        body = self.props.copy()
        body.pop('bgpvpn_id')

        bgpvpn_id = self.client_plugin().find_resourceid_by_name_or_id(
            'bgpvpn', self.props['bgpvpn_id'])

        net_assoc = self.neutron().create_network_association(
            bgpvpn_id,
            {'network_association': body})
        self.resource_id_set(net_assoc['network_association']['id'])

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        raise NotImplementedError()

    def handle_delete(self):
        try:
            self.neutron().delete_network_association(
                self.resource_id, self.properties['bgpvpn_id'])
        except Exception as ex:
            self.client_plugin().ignore_not_found(ex)
        else:
            return True

    def _confirm_delete(self):
        while True:
            try:
                self._show_resource()
            except exception.NotFound:
                return

    def _show_resource(self):
        return self.neutron().show_network_association(
            self.resource_id, self.properties['bgpvpn_id'])


class BGPVPNRouterAssoc(neutron.NeutronResource):

    """A resource for BGPVPNRouterAssoc in neutron.

    """

    PROPERTIES = (
        BGPVPN_ID, ROUTER_ID
    ) = (
        'bgpvpn_id', 'router_id'
    )

    ATTRIBUTES = (
        SHOW, STATUS
    ) = (
        'show', 'status'
    )

    properties_schema = {
        BGPVPN_ID: properties.Schema(
            properties.Schema.STRING,
            _('name or ID of the BGPVPN.'),
            required=True,
            constraints=[
                constraints.CustomConstraint('neutron.bgpvpn')
            ]
        ),
        ROUTER_ID: properties.Schema(
            properties.Schema.STRING,
            _('Router which shall be associated with the BGPVPN.'),
            required=True,
            constraints=[
                constraints.CustomConstraint('neutron.router')
            ]
        )
    }

    attributes_schema = {
        STATUS: attributes.Schema(
            _('Status of bgpvpn.'),
        ),
        SHOW: attributes.Schema(
            _('All attributes.')
        ),
    }

    def validate(self):
        super(BGPVPNRouterAssoc, self).validate()

    def handle_create(self):
        self.props = self.prepare_properties(self.properties,
                                             self.physical_resource_name())

        body = self.props.copy()
        body.pop('bgpvpn_id')

        bgpvpn_id = self.client_plugin().find_resourceid_by_name_or_id(
            'bgpvpn', self.props['bgpvpn_id'])

        router_assoc = self.neutron().create_router_association(
            bgpvpn_id,
            {'router_association': body})
        self.resource_id_set(router_assoc['router_association']['id'])

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        raise NotImplementedError()

    def handle_delete(self):
        try:
            self.neutron().delete_router_association(
                self.resource_id, self.properties['bgpvpn_id'])
        except Exception as ex:
            self.client_plugin().ignore_not_found(ex)
        else:
            return True

    def _confirm_delete(self):
        while True:
            try:
                self._show_resource()
            except exception.NotFound:
                return

    def _show_resource(self):
        return self.neutron().show_router_association(
            self.resource_id, self.properties['bgpvpn_id'])


def resource_mapping():
    return {
        'OS::Neutron::BGPVPN': BGPVPN,
        'OS::Neutron::BGPVPN-NET-ASSOCIATION': BGPVPNNetAssoc,
        'OS::Neutron::BGPVPN-ROUTER-ASSOCIATION': BGPVPNRouterAssoc,
    }



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import workflows
from openstack_dashboard import api

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api


class UpdateAssociations(workflows.MembershipAction):
    def __init__(self, request, resource_type, *args, **kwargs):
        super(UpdateAssociations, self).__init__(request,
                                                 *args,
                                                 **kwargs)
        err_msg = _('Unable to retrieve %ss list. '
                    'Please try again later.') % resource_type
        context = args[0]

        default_role_field_name = self.get_default_role_field_name()
        self.fields[default_role_field_name] = forms.CharField(required=False)
        self.fields[default_role_field_name].initial = 'member'

        field_name = self.get_member_field_name('member')
        self.fields[field_name] = forms.MultipleChoiceField(required=False)

        all_resources = self._get_resources(request, context, resource_type,
                                            err_msg)

        resources_list = [(resource.id, resource.name)
                          for resource in all_resources]

        self.fields[field_name].choices = resources_list

        bgpvpn_id = context.get('bgpvpn_id')

        try:
            if bgpvpn_id:
                associations = []
                list_method = getattr(bgpvpn_api, '%s_association_list' %
                                      resource_type)
                associations = [
                    getattr(association, '%s_id' %
                            resource_type) for association in
                    list_method(request, bgpvpn_id)
                    ]

        except Exception:
            exceptions.handle(request, err_msg)

        self.fields[field_name].initial = associations

    def _get_resources(self, request, context, resource_type, err_msg):
        """Get list of available resources."""
        # when an admin user uses the project panel BGPVPN, there is no
        # tenant_id in context because bgpvpn_get doesn't return it
        if request.user.is_superuser and context.get('tenant_id'):
            tenant_id = context.get('tenant_id')
        else:
            tenant_id = self.request.user.tenant_id
        try:
            if resource_type == 'router':
                return api.neutron.router_list(request, tenant_id=tenant_id)
            elif resource_type == 'network':
                return api.neutron.network_list_for_tenant(request, tenant_id)
            else:
                raise Exception(
                    _('Resource type %s is not supported') % resource_type)
        except Exception:
            exceptions.handle(request, err_msg % resource_type)


class UpdateBgpVpnRoutersAction(UpdateAssociations):
    def __init__(self, request, *args, **kwargs):
        super(UpdateBgpVpnRoutersAction, self).__init__(request,
                                                        'router',
                                                        *args,
                                                        **kwargs)

    class Meta(object):
        name = _("Router Associations")
        slug = "update_bgpvpn_router"


class UpdateBgpVpnRouters(workflows.UpdateMembersStep):
    action_class = UpdateBgpVpnRoutersAction
    help_text = _("Select the routers to be associated.")
    available_list_title = _("All Routers")
    members_list_title = _("Selected Routers")
    no_available_text = _("No router found.")
    no_members_text = _("No router selected.")
    show_roles = False
    depends_on = ("bgpvpn_id", "name")
    contributes = ("routers_association",)

    def contribute(self, data, context):
        if data:
            member_field_name = self.get_member_field_name('member')
            context['routers_association'] = data.get(member_field_name, [])
        return context


class UpdateBgpVpnNetworksAction(UpdateAssociations):
    def __init__(self, request, *args, **kwargs):
        super(UpdateBgpVpnNetworksAction, self).__init__(request,
                                                         'network',
                                                         *args,
                                                         **kwargs)

    class Meta(object):
        name = _("Network Associations")
        slug = "update_bgpvpn_network"


class UpdateBgpVpnNetworks(workflows.UpdateMembersStep):
    action_class = UpdateBgpVpnNetworksAction
    help_text = _("Select the networks to be associated.")
    available_list_title = _("All Networks")
    members_list_title = _("Selected Networks")
    no_available_text = _("No network found.")
    no_members_text = _("No network selected.")
    show_roles = False
    depends_on = ("bgpvpn_id", "name")
    contributes = ("networks_association",)

    def contribute(self, data, context):
        if data:
            member_field_name = self.get_member_field_name('member')
            context['networks_association'] = data.get(member_field_name, [])
        return context


class UpdateBgpVpnAssociations(workflows.Workflow):
    slug = "update_bgpvpn_associations"
    name = _("Edit BGPVPN associations")
    finalize_button_name = _("Save")
    success_message = _('Modified BGPVPN associations "%s".')
    failure_message = _('Unable to modify BGPVPN associations "%s".')
    success_url = "horizon:project:bgpvpn:index"
    default_steps = (UpdateBgpVpnNetworks,
                     UpdateBgpVpnRouters)

    def format_status_message(self, message):
        return message % self.context['name']

    def _handle_type(self, request, data, association_type):
        list_method = getattr(bgpvpn_api,
                              '%s_association_list' % association_type)
        associations = data["%ss_association" % association_type]
        try:
            old_associations = [
                getattr(association,
                        '%s_id' % association_type) for association in
                list_method(request, data['bgpvpn_id'])]
        except Exception:
            exceptions.handle(request,
                              _('Unable to retrieve %ss associations') %
                              association_type)
            raise

        to_remove_associations = list(set(old_associations) -
                                      set(associations))
        to_add_associations = list(set(associations) -
                                   set(old_associations))

        # If new resource added to the list
        if len(to_add_associations) > 0:
            for resource in to_add_associations:
                try:
                    create_asso = getattr(bgpvpn_api,
                                          '%s_association_create' %
                                          association_type)
                    params = self._set_params(data, association_type, resource)
                    create_asso(request,
                                data['bgpvpn_id'],
                                **params)
                except Exception as e:
                    exceptions.handle(
                        request,
                        _('Unable to associate {} {} {}').format(
                            association_type,
                            resource, str(e)))
                    raise

        # If resource has been deleted from the list
        if len(to_remove_associations) > 0:
            for resource in to_remove_associations:
                try:
                    list_method = getattr(bgpvpn_api,
                                          '%s_association_list' %
                                          association_type)
                    asso_list = list_method(request, data['bgpvpn_id'])
                    for association in asso_list:
                        if getattr(association,
                                   '%s_id' % association_type) == resource:
                            delete_method = getattr(bgpvpn_api,
                                                    '%s_association_delete' %
                                                    association_type)
                            delete_method(request,
                                          association.id, data['bgpvpn_id'])
                except Exception:
                    exceptions.handle(
                        request,
                        _('Unable to disassociate {}s {}').format(
                            association_type,
                            resource))
                    raise

    def _set_params(self, data, association_type, resource):
        params = dict()
        params['%s_id' % association_type] = resource
        return params

    def handle(self, request, data):
        action = False
        try:
            if 'networks_association' in data:
                self._handle_type(request, data, 'network')
                action = True
            if 'routers_association' in data:
                self._handle_type(request, data, 'router')
                action = True
            if not action:
                raise Exception(_('Associations type is not supported'))
        except Exception:
            return False
        return True



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from openstack_dashboard.api import neutron

LOG = logging.getLogger(__name__)

neutronclient = neutron.neutronclient


class Bgpvpn(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron bgpvpn."""


class NetworkAssociation(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron bgpvpn networks associations."""


class RouterAssociation(neutron.NeutronAPIDictWrapper):
    """Wrapper for neutron bgpvpn routers associations."""


def bgpvpns_list(request, **kwargs):
    LOG.debug("bgpvpn_list(): params=%s", kwargs)
    bgpvpns = neutronclient(request).list_bgpvpns(**kwargs).get('bgpvpns')
    return [Bgpvpn(v) for v in bgpvpns]


def bgpvpn_get(request, bgpvpn_id, **kwargs):
    LOG.debug("bgpvpn_get(): bgpvpnid=%s, kwargs=%s", bgpvpn_id, kwargs)
    bgpvpn = neutronclient(request).show_bgpvpn(bgpvpn_id,
                                                **kwargs).get('bgpvpn')
    return Bgpvpn(bgpvpn)


def bgpvpn_create(request, **kwargs):
    LOG.debug("bgpvpn_create(): params=%s", kwargs)
    body = {'bgpvpn': kwargs}
    bgpvpn = neutronclient(request).create_bgpvpn(body=body)
    return Bgpvpn(bgpvpn)


def bgpvpn_update(request, bgpvpn_id, **kwargs):
    LOG.debug("bgpvpn_update(): bgpvpnid=%s, kwargs=%s", bgpvpn_id, kwargs)
    body = {'bgpvpn': kwargs}
    bgpvpn = neutronclient(request).update_bgpvpn(bgpvpn_id,
                                                  body=body).get('bgpvpn')
    return Bgpvpn(bgpvpn)


def bgpvpn_delete(request, bgpvpn_id):
    LOG.debug("bgpvpn_delete(): bgpvpnid=%s", bgpvpn_id)
    neutronclient(request).delete_bgpvpn(bgpvpn_id)


def network_association_list(request, bgpvpn_id, **kwargs):
    LOG.debug("network_association_list(): bgpvpn_id=%s, kwargs=%s",
              bgpvpn_id, kwargs)
    network_associations = neutronclient(
        request).list_network_associations(
        bgpvpn_id,
        **kwargs).get('network_associations')
    return [NetworkAssociation(v) for v in network_associations]


def network_association_create(request, bgpvpn_id, **kwargs):
    LOG.debug("network_association_create(): bgpvpnid=%s kwargs=%s",
              bgpvpn_id, kwargs)
    body = {'network_association': kwargs}
    network_association = neutronclient(
        request).create_network_association(bgpvpn_id, body=body)
    return NetworkAssociation(network_association)


def network_association_delete(request, resource_id, bgpvpn_id):
    LOG.debug("network_association_delete(): resource_id=%s bgpvpnid=%s",
              resource_id, bgpvpn_id)
    neutronclient(request).delete_network_association(resource_id, bgpvpn_id)


def router_association_list(request, bgpvpn_id, **kwargs):
    LOG.debug("router_association_list(): bgpvpn_id=%s, kwargs=%s",
              bgpvpn_id, kwargs)
    router_associations = neutronclient(
        request).list_router_associations(bgpvpn_id,
                                          **kwargs).get('router_associations')
    return [RouterAssociation(v) for v in router_associations]


def router_association_create(request, bgpvpn_id, **kwargs):
    LOG.debug("router_association_create(): bgpvpnid=%s params=%s",
              bgpvpn_id, kwargs)
    body = {'router_association': kwargs}
    router_associations = neutronclient(
        request).create_router_association(bgpvpn_id, body=body)
    return RouterAssociation(router_associations)


def router_association_delete(request, resource_id, bgpvpn_id):
    LOG.debug("router_association_delete(): resource_id=%s bgpvpnid=%s",
              resource_id, bgpvpn_id)
    neutronclient(request).delete_router_association(resource_id, bgpvpn_id)



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from bgpvpn_dashboard.dashboards.project.bgpvpn import workflows \
    as project_workflows


class UpdateBgpVpnRouters(project_workflows.UpdateBgpVpnRouters):
    action_class = project_workflows.UpdateBgpVpnRoutersAction
    depends_on = ("bgpvpn_id", "tenant_id", "name")


class UpdateBgpVpnNetworks(project_workflows.UpdateBgpVpnNetworks):
    action_class = project_workflows.UpdateBgpVpnNetworksAction
    depends_on = ("bgpvpn_id", "tenant_id", "name")


class UpdateBgpVpnAssociations(project_workflows.UpdateBgpVpnAssociations):
    success_url = "horizon:admin:bgpvpn:index"
    default_steps = (UpdateBgpVpnNetworks,
                     UpdateBgpVpnRouters)

    def _set_params(self, data, association_type, resource):
        params = super(
            UpdateBgpVpnAssociations, self)._set_params(data,
                                                        association_type,
                                                        resource)
        params['tenant_id'] = data['tenant_id']
        return params



# Copyright (c) 2017 Orange.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api
from bgpvpn_dashboard.dashboards.project.bgpvpn import forms as bgpvpn_form
from bgpvpn_dashboard.dashboards.project.bgpvpn import tables as bgpvpn_tables
from bgpvpn_dashboard.dashboards.project.bgpvpn import views as bgpvpn_views
from bgpvpn_dashboard.dashboards.project.bgpvpn import workflows as \
    bgpvpn_workflows
from openstack_dashboard.test import helpers


class TestIndexView(helpers.APITestCase):

    def setUp(self):
        super(TestIndexView, self).setUp()
        mock_request = mock.Mock(horizon={'async_messages': []})
        self.bgpvpn_view = bgpvpn_views.IndexView(request=mock_request)
        self.bgpvpn_view._prev = False
        self.bgpvpn_view._more = False

        self.assertEqual(bgpvpn_tables.BgpvpnTable,
                         self.bgpvpn_view.table_class)
        self.assertEqual('project/bgpvpn/index.html',
                         self.bgpvpn_view.template_name)

    def _get_mock_bgpvpn(self, prefix):
        bgpvpn_info = {}
        if prefix:
            bgpvpn_info = {
                "name": "%s_name" % prefix,
                "route_targets": [],
                "import_targets": [],
                "export_targets": [],
                "networks": [],
                "routers": [],
                "tenant_id": "tenant_id",
                "type": "l3"
            }
        return bgpvpn_api.Bgpvpn(bgpvpn_info)

    @mock.patch.object(bgpvpn_views, 'bgpvpn_api', autospec=True)
    def test_get_data(self, mock_bgpvpn_api):
        """Test that get_data works."""

        bgpvpn_foo = self._get_mock_bgpvpn("foo")
        bgpvpn_bar = self._get_mock_bgpvpn("bar")

        mock_bgpvpn_api.bgpvpns_list.return_value = [bgpvpn_bar, bgpvpn_foo]
        result = self.bgpvpn_view.get_data()
        expected_bgpvpns = [bgpvpn_bar, bgpvpn_foo]
        self.assertEqual(expected_bgpvpns, result)


class TestEditDataView(helpers.APITestCase):

    def setUp(self):
        super(TestEditDataView, self).setUp()
        mock_request = mock.Mock(horizon={'async_messages': []})
        self.bgpvpn_view = bgpvpn_views.EditDataView(request=mock_request)
        fake_response = {'status_code': 200}
        self.mock_request = mock.Mock(return_value=fake_response, META=[])
        self.bgpvpn_view.request = self.mock_request
        self.bgpvpn_view.kwargs = {'bgpvpn_id': 'foo-id'}

        self.assertEqual(bgpvpn_form.EditDataBgpVpn,
                         self.bgpvpn_view.form_class)
        self.assertEqual('horizon:project:bgpvpn:edit',
                         self.bgpvpn_view.submit_url)
        self.assertEqual(reverse_lazy('horizon:project:bgpvpn:index'),
                         self.bgpvpn_view.success_url)
        self.assertEqual('project/bgpvpn/modify.html',
                         self.bgpvpn_view.template_name)

    @mock.patch.object(bgpvpn_views, 'bgpvpn_api', autospec=True)
    def test_get_initial(self, mock_bgpvpn_api):
        self.bgpvpn_view.request.user.is_superuser = False
        bgpvpn_data = {"name": "foo-name",
                       "id": "foo-id",
                       "type": "l3"}
        expected_data = {"name": "foo-name",
                         "bgpvpn_id": "foo-id",
                         "type": "l3"}
        mock_bgpvpn_api.bgpvpn_get.return_value = bgpvpn_api.Bgpvpn(
            bgpvpn_data)
        result = self.bgpvpn_view.get_initial()

        mock_bgpvpn_api.bgpvpn_get.assert_called_once_with(
            self.bgpvpn_view.request, "foo-id")
        for key, val in expected_data.items():
            self.assertEqual(val, result[key])

    def test_get_context_data(self):
        mock_form = mock.Mock()
        args = ("foo-id",)
        expected_context = {
            'bgpvpn_id': 'foo-id',
            'submit_url': reverse("horizon:project:bgpvpn:edit", args=args)}
        context = self.bgpvpn_view.get_context_data(form=mock_form)
        self.assertIn('view', context)
        self.assertIsInstance(context['view'], bgpvpn_views.EditDataView)

        for key, val in expected_context.items():
            self.assertIn(key, context)

            self.assertEqual(val, context[key])

    def test_join_rts(self):
        route_targets_list = ["65400:1", "65401:1"]
        expected_result = "65400:1,65401:1"
        result = self.bgpvpn_view._join_rts(route_targets_list)
        self.assertEqual(expected_result, result)


class TestUpdateAssociationsView(helpers.APITestCase):

    def setUp(self):
        super(TestUpdateAssociationsView, self).setUp()
        self.bgpvpn_view = bgpvpn_views.UpdateAssociationsView()
        fake_response = {'status_code': 200}
        self.mock_request = mock.Mock(return_value=fake_response, META=[])
        self.bgpvpn_view.request = self.mock_request
        self.bgpvpn_view.kwargs = {'bgpvpn_id': 'foo-id'}

        self.assertEqual(bgpvpn_workflows.UpdateBgpVpnAssociations,
                         self.bgpvpn_view.workflow_class)
        self.assertEqual(reverse_lazy("horizon:project:bgpvpn:index"),
                         self.bgpvpn_view.failure_url)

    @mock.patch.object(bgpvpn_views, 'bgpvpn_api', autospec=True)
    def test_get_initial(self, mock_bgpvpn_api):
        bgpvpn_data = {"name": "foo-name",
                       "id": "foo-id",
                       "type": "l3"}
        expected_data = {"name": "foo-name",
                         "bgpvpn_id": "foo-id",
                         "type": "l3"}
        mock_bgpvpn_api.bgpvpn_get.return_value = bgpvpn_api.Bgpvpn(
            bgpvpn_data)
        result = self.bgpvpn_view.get_initial()

        mock_bgpvpn_api.bgpvpn_get.assert_called_once_with(
            self.bgpvpn_view.request, "foo-id")
        for key, val in expected_data.items():
            self.assertEqual(val, result[key])


class TestDetailProjectView(helpers.APITestCase):

    def setUp(self):
        super(TestDetailProjectView, self).setUp()

        self.view = bgpvpn_views.DetailProjectView()
        fake_response = {'status_code': 200}
        self.mock_request = mock.Mock(return_value=fake_response)
        self.view.request = self.mock_request
        self.view.kwargs = {'bgpvpn_id': 'foo-id'}

        self.assertEqual('project/bgpvpn/detail.html', self.view.template_name)
        self.assertEqual('{{ bgpvpn.name }}', self.view.page_title)
        self.assertEqual('horizon:project:bgpvpn:index',
                         self.view.redirect_url)
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(bgpvpn_views, 'bgpvpn_api', autospec=True)
    def test_get_context_data(self, mock_bgpvpn_api):
        expected_bgpvpn = bgpvpn_api.Bgpvpn({"id": "foo-id"})
        mock_bgpvpn_api.bgpvpn_get.return_value = expected_bgpvpn
        context = self.view.get_context_data()

        mock_bgpvpn_api.bgpvpn_get.assert_called_once_with(
            self.view.request, "foo-id")
        self.assertIn('view', context)
        self.assertIn('bgpvpn', context)
        self.assertIn('url', context)
        self.assertIsInstance(context['view'], bgpvpn_views.DetailProjectView)
        self.assertEqual(expected_bgpvpn, context['bgpvpn'])
        self.assertEqual(reverse('horizon:project:bgpvpn:index'),
                         context['url'])



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.core.urlresolvers import reverse
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon.utils import memoized
from horizon import views
from horizon import workflows
from openstack_dashboard import api

import bgpvpn_dashboard.api.bgpvpn as bgpvpn_api
from bgpvpn_dashboard.common import bgpvpn as bgpvpn_common
from bgpvpn_dashboard.dashboards.project.bgpvpn import forms as bgpvpn_forms
from bgpvpn_dashboard.dashboards.project.bgpvpn import tables as bgpvpn_tables
from bgpvpn_dashboard.dashboards.project.bgpvpn import workflows \
    as bgpvpn_workflows


class IndexView(tables.DataTableView):
    table_class = bgpvpn_tables.BgpvpnTable
    template_name = 'project/bgpvpn/index.html'
    page_title = _("BGP VPNs")

    @memoized.memoized_method
    def get_data(self):
        tenant_id = self.request.user.tenant_id
        bgpvpns_list = bgpvpn_api.bgpvpns_list(
            self.request, tenant_id=tenant_id)
        for bgpvpn in bgpvpns_list:
            bgpvpn.networks = [api.neutron.network_get(
                self.request, id, expand_subnet=False)
                for id in bgpvpn.networks]
            bgpvpn.routers = [api.neutron.router_get(self.request, id)
                              for id in bgpvpn.routers]
        return bgpvpns_list


class EditDataView(forms.ModalFormView):
    form_class = bgpvpn_forms.EditDataBgpVpn
    form_id = "edit_data_bgpvpn_form"
    modal_header = _("Edit BGPVPN")
    submit_label = _("Update Change")
    submit_url = 'horizon:project:bgpvpn:edit'
    success_url = reverse_lazy('horizon:project:bgpvpn:index')
    template_name = 'project/bgpvpn/modify.html'
    page_title = _("Edit BGPVPN")

    @staticmethod
    def _join_rts(route_targets_list):
        return ','.join(route_targets_list)

    def get_context_data(self, **kwargs):
        context = super(EditDataView, self).get_context_data(**kwargs)
        args = (self.kwargs['bgpvpn_id'],)
        context["bgpvpn_id"] = self.kwargs['bgpvpn_id']
        context["submit_url"] = reverse(self.submit_url, args=args)
        return context

    def get_initial(self):
        bgpvpn_id = self.kwargs['bgpvpn_id']
        try:
            # Get initial bgpvpn information
            bgpvpn = bgpvpn_api.bgpvpn_get(self.request, bgpvpn_id)
        except Exception:
            exceptions.handle(
                self.request,
                _('Unable to retrieve BGPVPN details.'),
                redirect=self.success_url)
        else:
            data = bgpvpn.to_dict()
            if self.request.user.is_superuser:
                for attribute in bgpvpn_common.RT_FORMAT_ATTRIBUTES:
                    data[attribute] = self._join_rts(bgpvpn[attribute])
            data['bgpvpn_id'] = data.pop('id')
            return data


class UpdateAssociationsView(workflows.WorkflowView):
    workflow_class = bgpvpn_workflows.UpdateBgpVpnAssociations
    page_title = _("Edit BGPVPN associations")
    failure_url = reverse_lazy("horizon:project:bgpvpn:index")

    def get_initial(self):
        bgpvpn_id = self.kwargs['bgpvpn_id']
        try:
            # Get initial bgpvpn information
            bgpvpn = bgpvpn_api.bgpvpn_get(self.request, bgpvpn_id)
            data = bgpvpn.to_dict()
            data['bgpvpn_id'] = data.pop('id')
            return data
        except Exception:
            exceptions.handle(
                self.request,
                _('Unable to retrieve BGPVPN details.'),
                redirect=self.failure_url)


class DetailProjectView(views.HorizonTemplateView):
    template_name = 'project/bgpvpn/detail.html'
    page_title = "{{ bgpvpn.name }}"
    redirect_url = 'horizon:project:bgpvpn:index'

    def get_context_data(self, **kwargs):
        context = super(DetailProjectView, self).get_context_data(**kwargs)
        bgpvpn = self.get_data()
        table = bgpvpn_tables.BgpvpnTable(self.request)
        context["bgpvpn"] = bgpvpn
        context["url"] = reverse(self.redirect_url)
        context["actions"] = table.render_row_actions(bgpvpn)
        return context

    @memoized.memoized_method
    def get_data(self):
        try:
            bgpvpn_id = self.kwargs['bgpvpn_id']
            return bgpvpn_api.bgpvpn_get(self.request, bgpvpn_id)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve BGPVPN details.'),
                              redirect=reverse(self.redirect_url))



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf.urls import url

import bgpvpn_dashboard.dashboards.admin.bgpvpn.views as bgpvpn_views

BGPVPN = r'^(?P<bgpvpn_id>[^/]+)/%s$'

urlpatterns = [
    url(r'^$', bgpvpn_views.IndexView.as_view(), name='index'),
    url(r'^create/$', bgpvpn_views.CreateView.as_view(), name='create'),
    url(BGPVPN % 'edit', bgpvpn_views.EditDataView.as_view(), name='edit'),
    url(BGPVPN % 'update-associations',
        bgpvpn_views.UpdateAssociationsView.as_view(),
        name='update-associations'),
    url(r'^(?P<bgpvpn_id>[^/]+)/detail/$',
        bgpvpn_views.DetailProjectView.as_view(), name='detail'),
]



# Copyright (c) 2015 Ericsson.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base

# This is the representation of the bgpvpn
# client of networking-bgpvpn

BGPVPN_OBJECT_PATH = '/bgpvpn/bgpvpns'
BGPVPN_RESOURCE_PATH = '/bgpvpn/bgpvpns/%s'
BGPVPN_NETWORK_ASSOCIATION_PATH = '/bgpvpn/bgpvpns/%s/network_associations'
BGPVPN_ROUTER_ASSOCIATION_PATH = '/bgpvpn/bgpvpns/%s/router_associations'


class BgpvpnClient(base.BaseNetworkClient):

    def create_bgpvpn(self, **kwargs):
        uri = BGPVPN_OBJECT_PATH
        post_data = {'bgpvpn': kwargs}
        return self.create_resource(uri, post_data)

    def delete_bgpvpn(self, bgpvpn_id):
        uri = BGPVPN_RESOURCE_PATH % bgpvpn_id
        return self.delete_resource(uri)

    def show_bgpvpn(self, bgpvpn_id, **fields):
        uri = BGPVPN_RESOURCE_PATH % bgpvpn_id
        return self.show_resource(uri, **fields)

    def list_bgpvpns(self, **filters):
        uri = BGPVPN_OBJECT_PATH
        return self.list_resources(uri, **filters)

    def update_bgpvpn(self, bgpvpn_id, **kwargs):
        uri = BGPVPN_RESOURCE_PATH % bgpvpn_id
        post_data = {'bgpvpn': kwargs}
        return self.update_resource(uri, post_data)

    def create_network_association(self, bgpvpn_id, network_id):
        uri = BGPVPN_NETWORK_ASSOCIATION_PATH % bgpvpn_id
        post_data = {"network_association":
                     {"network_id": network_id}}
        return self.create_resource(uri, post_data)

    def delete_network_association(self, bgpvpn_id, association_id):
        uri_pattern = BGPVPN_NETWORK_ASSOCIATION_PATH + "/%s"
        uri = uri_pattern % (bgpvpn_id, association_id)
        return self.delete_resource(uri)

    def show_network_association(self, bgpvpn_id, association_id, **fields):
        uri_pattern = BGPVPN_NETWORK_ASSOCIATION_PATH + "/%s"
        uri = uri_pattern % (bgpvpn_id, association_id)
        return self.show_resource(uri, **fields)

    def list_network_associations(self, bgpvpn_id, **filters):
        uri = BGPVPN_NETWORK_ASSOCIATION_PATH % bgpvpn_id
        return self.list_resources(uri, **filters)

    def create_router_association(self, bgpvpn_id, router_id):
        uri = BGPVPN_ROUTER_ASSOCIATION_PATH % bgpvpn_id
        post_data = {"router_association":
                     {"router_id": router_id}}
        return self.create_resource(uri, post_data)

    def delete_router_association(self, bgpvpn_id, association_id):
        uri_pattern = BGPVPN_ROUTER_ASSOCIATION_PATH + "/%s"
        uri = uri_pattern % (bgpvpn_id, association_id)
        return self.delete_resource(uri)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import copy

from bgpvpn_dashboard.api import bgpvpn
from openstack_dashboard.test.test_data import utils


def data(TEST):
    TEST.bgpvpns = utils.TestDataContainer()
    TEST.api_bgpvpns = utils.TestDataContainer()
    TEST.network_associations = utils.TestDataContainer()
    TEST.api_network_associations = utils.TestDataContainer()
    TEST.router_associations = utils.TestDataContainer()
    TEST.api_router_associations = utils.TestDataContainer()

    bgpvpn_dict = {'id': 'b595e758-1877-4aec-92a2-6834d76f1025',
                   'tenant_id': '1',
                   'name': 'bgpvpn1',
                   'route_targets': '64500:1'
                   }
    TEST.api_bgpvpns.add(bgpvpn_dict)
    b = bgpvpn.Bgpvpn(copy.deepcopy(bgpvpn_dict))
    TEST.bgpvpns.add(b)

    network_association_dict = {
        'id': '99ef096d-21fb-43a7-9e2a-b3c464abef3a',
        'network_id': '063cf7f3-ded1-4297-bc4c-31eae876cc91',
        'tenant_id': '1'}
    TEST.api_network_associations.add(network_association_dict)
    na = bgpvpn.NetworkAssociation(copy.deepcopy(network_association_dict))
    TEST.network_associations.add(na)

    router_association_dict = {
        'id': '9736c228-745d-4e78-83a5-d971d9fd8f2c',
        'router_id': '279989f7-54bb-41d9-ba42-0d61f12fda61',
        'tenant_id': '1'}
    TEST.api_router_associations.add(router_association_dict)
    ra = bgpvpn.RouterAssociation(copy.deepcopy(router_association_dict))
    TEST.router_associations.add(ra)



#
# Copyright (C) 2015 Ericsson India Global Services Pvt Ltd.
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.
#

import mock


from networking_bgpvpn.tests.unit.services import test_plugin


class TestBgpvpnOdlCommon(test_plugin.BgpvpnTestCaseMixin):

    def setUp(self):
        self.mocked_odlclient = mock.patch(
            'networking_odl.common.client'
            '.OpenDaylightRestClient.create_client').start().return_value

        provider = ('networking_bgpvpn.neutron.services.service_drivers.'
                    'opendaylight.odl.OpenDaylightBgpvpnDriver')
        super(TestBgpvpnOdlCommon, self).setUp(service_provider=provider)


class TestOdlServiceDriver(TestBgpvpnOdlCommon):

    def test_odl_associate_net(self):
        mocked_sendjson = self.mocked_odlclient.sendjson
        with self.port() as port1:
            net_id = port1['port']['network_id']
            with self.bgpvpn() as bgpvpn:
                id = bgpvpn['bgpvpn']['id']
                rt = bgpvpn['bgpvpn']['route_targets']
                mocked_sendjson.reset_mock()
                with self.assoc_net(id, net_id):
                    formatted_bgpvpn = {
                        'bgpvpn':
                        {'export_targets': mock.ANY,
                         'name': mock.ANY,
                         'route_targets': rt,
                         'tenant_id': mock.ANY,
                         'project_id': mock.ANY,
                         'import_targets': mock.ANY,
                         'route_distinguishers': mock.ANY,
                         'type': mock.ANY,
                         'id': id,
                         'networks': [net_id],
                         'routers': []}}
                    mocked_sendjson.assert_called_once_with(mock.ANY,
                                                            mock.ANY,
                                                            formatted_bgpvpn)

    def test_odl_associate_router(self):
        mocked_sendjson = self.mocked_odlclient.sendjson
        with self.router(tenant_id=self._tenant_id) as router:
            router_id = router['router']['id']
            with self.bgpvpn() as bgpvpn:
                id = bgpvpn['bgpvpn']['id']
                rt = bgpvpn['bgpvpn']['route_targets']
                mocked_sendjson.reset_mock()
                with self.assoc_router(id, router_id):
                    formatted_bgpvpn = {
                        'bgpvpn':
                        {'export_targets': mock.ANY,
                         'name': mock.ANY,
                         'route_targets': rt,
                         'tenant_id': mock.ANY,
                         'project_id': mock.ANY,
                         'import_targets': mock.ANY,
                         'route_distinguishers': mock.ANY,
                         'type': mock.ANY,
                         'id': id,
                         'networks': [],
                         'routers': [router_id]}}
                    mocked_sendjson.assert_called_once_with(mock.ANY,
                                                            mock.ANY,
                                                            formatted_bgpvpn)



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import forms
from horizon.utils import memoized

from openstack_dashboard import api

import bgpvpn_dashboard.api.bgpvpn as bgpvpn_api
from bgpvpn_dashboard.dashboards.admin.bgpvpn import forms as bgpvpn_forms
from bgpvpn_dashboard.dashboards.admin.bgpvpn import tables as bgpvpn_tables
from bgpvpn_dashboard.dashboards.admin.bgpvpn import workflows \
    as bgpvpn_workflows
from bgpvpn_dashboard.dashboards.project.bgpvpn import views as project_views


class IndexView(project_views.IndexView):
    table_class = bgpvpn_tables.BgpvpnTable

    @memoized.memoized_method
    def _get_tenant_name(self, id):
        try:
            return api.keystone.tenant_get(self.request, id).name
        except Exception:
            msg = _("Unable to retrieve information about the "
                    "tenant %s") % id
            exceptions.handle(self.request, msg)

    def get_data(self):
        bgpvpns_list = bgpvpn_api.bgpvpns_list(self.request)
        for bgpvpn in bgpvpns_list:
            bgpvpn.networks = [api.neutron.network_get(
                self.request, id, expand_subnet=False)
                for id in bgpvpn.networks]
            bgpvpn.routers = [api.neutron.router_get(self.request, id)
                              for id in bgpvpn.routers]
            bgpvpn.tenant_name = self._get_tenant_name(bgpvpn.tenant_id)
        return bgpvpns_list


class CreateView(forms.ModalFormView):
    form_class = bgpvpn_forms.CreateBgpVpn
    form_id = "create_bgpvpn_form"
    template_name = 'admin/bgpvpn/create.html'
    submit_label = _("Create BGPVPN")
    success_url = reverse_lazy('horizon:admin:bgpvpn:index')
    page_title = _("Create BGPVPN")
    submit_url = reverse_lazy("horizon:admin:bgpvpn:create")


class EditDataView(project_views.EditDataView):
    form_class = bgpvpn_forms.EditDataBgpVpn
    submit_url = 'horizon:admin:bgpvpn:edit'
    success_url = reverse_lazy('horizon:admin:bgpvpn:index')


class UpdateAssociationsView(project_views.UpdateAssociationsView):
    workflow_class = bgpvpn_workflows.UpdateBgpVpnAssociations
    page_title = _("Edit BGPVPN associations")
    failure_url = reverse_lazy("horizon:admin:bgpvpn:index")


class DetailProjectView(project_views.DetailProjectView):
    template_name = 'admin/bgpvpn/detail.html'
    redirect_url = 'horizon:admin:bgpvpn:index'



# Copyright (c) 2015 Cloudwatt.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from oslo_log import log
from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from neutron_lib import exceptions as n_exc

from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
from networking_bgpvpn.neutron.services.common import constants
from networking_bgpvpn.neutron.services.common import utils
from networking_bgpvpn.neutron.services.service_drivers import driver_api
from networking_bgpvpn.neutron.services.service_drivers.opencontrail import \
    exceptions as oc_exc
from networking_bgpvpn.neutron.services.service_drivers.opencontrail import \
    opencontrail_client

OPENCONTRAIL_BGPVPN_DRIVER_NAME = 'OpenContrail'

LOG = log.getLogger(__name__)


class OpenContrailBGPVPNDriver(driver_api.BGPVPNDriverBase):
    """BGP VPN Service Driver class for OpenContrail."""

    def __init__(self, service_plugin):
        super(OpenContrailBGPVPNDriver, self).__init__(service_plugin)
        LOG.debug("OpenContrailBGPVPNDriver service_plugin : %s",
                  service_plugin)

    def _get_opencontrail_api_client(self, context):
        return opencontrail_client.OpenContrailAPIBaseClient()

    def _locate_rt(self, oc_client, rt_fq_name):
        try:
            rt_uuid = oc_client.fqname_to_id('route-target', rt_fq_name)
        except oc_exc.OpenContrailAPINotFound:
            body = {
                'route-target': {
                    "fq_name": [':'.join(rt_fq_name)]
                }
            }
            rt_uuid = oc_client.create('Route Target', body)['uuid']

        return rt_uuid

    def _clean_route_targets(self, oc_client, rts_fq_name):
        for rt_fq_name in rts_fq_name:
            try:
                rt_uuid = oc_client.fqname_to_id('route-target', rt_fq_name)
            except oc_exc.OpenContrailAPINotFound:
                continue

            rt = oc_client.show('Route Target', rt_uuid)
            if 'routing_instance_back_refs' not in rt.keys():
                # rt not use anymore, remove it
                rt = oc_client.remove('Route Target', rt_uuid)

    def _update_rt_ri_association(self, oc_client, operation, ri_id,
                                  rt_fq_name, import_export=None):
        rt_uuid = self._locate_rt(oc_client, rt_fq_name)

        kwargs = {
            "operation": operation,
            "resource_type": "routing-instance",
            "resource_uuid": ri_id,
            "ref_type": "route-target",
            "ref_fq_name": rt_fq_name,
            "ref_uuid": rt_uuid,
            "attributes": {
                "import_export": import_export
            }
        }
        oc_client.ref_update(**kwargs)

        if operation == 'DELETE':
            self._clean_route_targets(oc_client, [rt_fq_name])

    def _get_ri_id_of_network(self, oc_client, network_id):
        try:
            network = oc_client.show('Virtual Network', network_id)
            ri_fq_name = network['fq_name'] + [network['fq_name'][-1]]
            for ri_ref in network.get('routing_instances', []):
                if ri_ref['to'] == ri_fq_name:
                    return ri_ref['uuid']
        except (oc_exc.OpenContrailAPINotFound, IndexError):
            raise n_exc.NetworkNotFound(net_id=network_id)
        raise n_exc.NetworkNotFound(net_id=network_id)

    def _set_bgpvpn_association(self, oc_client, operation, bgpvpn,
                                networks=None):
        if networks is None:
            networks = []
        for network_id in networks:
            try:
                net_ri_id = self._get_ri_id_of_network(oc_client, network_id)
            except n_exc.NetworkNotFound:
                LOG.info("Network %s not found, cleanup route targets",
                         network_id)
                rts_fq_name = (bgpvpn['route_targets'] +
                               bgpvpn['import_targets'] +
                               bgpvpn['export_targets'])
                rts_fq_name = [['target'] + rt.split(':') for rt in
                               rts_fq_name]
                self._clean_route_targets(oc_client, rts_fq_name)
                return bgpvpn
            if bgpvpn['type'] == constants.BGPVPN_L3:
                for rt in bgpvpn['route_targets']:
                    rt_fq_name = ['target'] + rt.split(':')
                    self._update_rt_ri_association(oc_client, operation,
                                                   net_ri_id, rt_fq_name)

                for rt in bgpvpn['import_targets']:
                    rt_fq_name = ['target'] + rt.split(':')
                    self._update_rt_ri_association(oc_client, operation,
                                                   net_ri_id, rt_fq_name,
                                                   import_export="import")

                for rt in bgpvpn['export_targets']:
                    rt_fq_name = ['target'] + rt.split(':')
                    self._update_rt_ri_association(oc_client, operation,
                                                   net_ri_id, rt_fq_name,
                                                   import_export="export")

        return bgpvpn

    # Check if tenant ID exists by reading it from the Contrail API;
    # if not, it sends an exception
    def _check_tenant_id(self, oc_client, tenant_id):
        try:
            tenant_id = str(uuid.UUID(tenant_id))
        except ValueError:
            raise oc_exc.OpenContrailMalformedUUID(uuid=tenant_id)
        oc_client.show('Project', tenant_id)

    def create_bgpvpn(self, context, bgpvpn):
        LOG.debug("create_bgpvpn_ called with %s" % bgpvpn)

        # Only support l3 technique
        if not bgpvpn['type']:
            bgpvpn['type'] = constants.BGPVPN_L3
        elif bgpvpn['type'] != constants.BGPVPN_L3:
            raise bgpvpn_ext.BGPVPNTypeNotSupported(
                driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME, type=bgpvpn['type'])

        # Does not support to set route distinguisher
        if 'route_distinguishers' in bgpvpn and bgpvpn['route_distinguishers']:
            raise bgpvpn_ext.BGPVPNRDNotSupported(
                driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)

        oc_client = self._get_opencontrail_api_client(context)

        # Check if tenant ID exists;
        # if not, it sends an exception
        self._check_tenant_id(oc_client, bgpvpn['tenant_id'])

        bgpvpn['id'] = uuidutils.generate_uuid()

        oc_client.kv_store('STORE', key=bgpvpn['id'], value={'bgpvpn': bgpvpn})

        return utils.make_bgpvpn_dict(bgpvpn)

    def get_bgpvpns(self, context, filters=None, fields=None):
        LOG.debug("get_bgpvpns called, fields = %s, filters = %s"
                  % (fields, filters))

        oc_client = self._get_opencontrail_api_client(context)

        bgpvpns = []
        for kv_dict in oc_client.kv_store('RETRIEVE'):
            try:
                value = jsonutils.loads(kv_dict['value'])
            except ValueError:
                continue
            if (isinstance(value, dict) and
                    'bgpvpn' in value and
                    utils.filter_resource(value['bgpvpn'], filters)):
                bgpvpn = value['bgpvpn']
                if not fields or 'networks' in fields:
                    bgpvpn['networks'] = \
                        [net_assoc['network_id'] for net_assoc in
                         self.get_net_assocs(context, bgpvpn['id'])]
                bgpvpns.append(utils.make_bgpvpn_dict(bgpvpn, fields))

        return bgpvpns

    def _clean_bgpvpn_assoc(self, oc_client, bgpvpn_id):
        for kv_dict in oc_client.kv_store('RETRIEVE'):
            try:
                value = jsonutils.loads(kv_dict['value'])
            except ValueError:
                continue
            if (isinstance(value, dict) and
                    'bgpvpn_net_assoc' in value and
                    value['bgpvpn_net_assoc']['bgpvpn_id'] == bgpvpn_id):
                assoc_id = value['bgpvpn_net_assoc']['id']
                oc_client.kv_store('DELETE', key=assoc_id)

    def get_bgpvpn(self, context, id, fields=None):
        LOG.debug("get_bgpvpn called for id %s with fields = %s"
                  % (id, fields))

        oc_client = self._get_opencontrail_api_client(context)

        try:
            bgpvpn = jsonutils.loads(oc_client.kv_store('RETRIEVE', key=id))
        except (oc_exc.OpenContrailAPINotFound, ValueError):
            raise bgpvpn_ext.BGPVPNNotFound(id=id)

        if (not isinstance(bgpvpn, dict) or 'bgpvpn' not in bgpvpn):
            raise bgpvpn_ext.BGPVPNNotFound(id=id)

        bgpvpn = bgpvpn['bgpvpn']
        if not fields or 'networks' in fields:
            bgpvpn['networks'] = [net_assoc['network_id'] for net_assoc in
                                  self.get_net_assocs(context, id)]
        return utils.make_bgpvpn_dict(bgpvpn, fields)

    def update_bgpvpn(self, context, id, new_bgpvpn):
        LOG.debug("update_bgpvpn called with %s for %s" % (new_bgpvpn, id))

        oc_client = self._get_opencontrail_api_client(context)

        old_bgpvpn = self.get_bgpvpn(context, id)
        networks = old_bgpvpn.get('networks', [])
        bgpvpn = old_bgpvpn.copy()
        bgpvpn.update(new_bgpvpn)

        (added_keys, removed_keys, changed_keys) = \
            utils.get_bgpvpn_differences(bgpvpn, old_bgpvpn)
        if not (added_keys or removed_keys or changed_keys):
            return utils.make_bgpvpn_dict(bgpvpn)

        # Does not support to update route distinguisher
        if 'route_distinguishers' in added_keys | removed_keys | changed_keys:
            raise bgpvpn_ext.BGPVPNRDNotSupported(
                driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)

        rt_keys = set(['route_targets',
                       'import_targets',
                       'export_targets'])

        if (rt_keys & added_keys or
                rt_keys & changed_keys or
                rt_keys & removed_keys):
            self._set_bgpvpn_association(oc_client, 'DELETE', old_bgpvpn,
                                         networks)
            self._set_bgpvpn_association(oc_client, 'ADD', bgpvpn, networks)

        oc_client.kv_store('STORE', key=id, value={'bgpvpn': bgpvpn})
        return utils.make_bgpvpn_dict(bgpvpn)

    def delete_bgpvpn(self, context, id):
        LOG.debug("delete_bgpvpn called for id %s" % id)

        bgpvpn = self.get_bgpvpn(context, id)
        networks = bgpvpn.get('networks', [])
        oc_client = self._get_opencontrail_api_client(context)

        self._set_bgpvpn_association(oc_client, 'DELETE', bgpvpn, networks)
        self._clean_bgpvpn_assoc(oc_client, id)
        oc_client.kv_store('DELETE', key=id)

    def create_net_assoc(self, context, bgpvpn_id, network_association):
        LOG.debug("create_net_assoc called for bgpvpn %s with network %s"
                  % (bgpvpn_id, network_association['network_id']))

        bgpvpn = self.get_bgpvpn(context, bgpvpn_id)
        oc_client = self._get_opencontrail_api_client(context)

        network_id = network_association['network_id']
        if network_id not in bgpvpn.get('networks', []):
            assoc_uuid = uuidutils.generate_uuid()
            self._set_bgpvpn_association(oc_client, 'ADD', bgpvpn,
                                         [network_id])

            assoc_dict = utils.make_net_assoc_dict(
                assoc_uuid, network_association['tenant_id'],
                bgpvpn_id, network_association['network_id'])
            oc_client.kv_store('STORE', key=assoc_uuid,
                               value={'bgpvpn_net_assoc': assoc_dict})
            return assoc_dict
        else:
            # the tuple (bgpvpn_id, network_id) is necessarily unique
            return self.get_net_assocs(context, bgpvpn_id,
                                       filters={'network_id': network_id})[0]

    def get_net_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
        LOG.debug("get_net_assoc called for %s for BGPVPN %s, with fields = %s"
                  % (assoc_id, bgpvpn_id, fields))

        oc_client = self._get_opencontrail_api_client(context)

        try:
            net_assoc = jsonutils.loads(
                oc_client.kv_store('RETRIEVE', key=assoc_id))
        except (oc_exc.OpenContrailAPINotFound, ValueError):
            raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                    bgpvpn_id=bgpvpn_id)

        if (not isinstance(net_assoc, dict) or
                'bgpvpn_net_assoc' not in net_assoc):
            raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                    bgpvpn_id=bgpvpn_id)
        net_assoc = net_assoc['bgpvpn_net_assoc']

        if net_assoc['bgpvpn_id'] != bgpvpn_id:
            raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                    bgpvpn_id=bgpvpn_id)

        # It the bgpvpn was deleted, the 'get_bgpvpn' will clean all related
        # associations and replaces BGPVPNNotFound by a BGPVPNNetAssocNotFound
        try:
            get_fields = ['tenant_id', 'route_targets', 'import_targets',
                          'export_targets']
            bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'],
                                     fields=get_fields)
        except bgpvpn_ext.BGPVPNNotFound:
            raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                    bgpvpn_id=bgpvpn_id)

        # If the network was delete all bgpvpn related association should be
        # deleted also
        try:
            oc_client.id_to_fqname(net_assoc['network_id'])
        except oc_exc.OpenContrailAPINotFound:
            self._set_bgpvpn_association(oc_client, 'DELETE', bgpvpn,
                                         [net_assoc['network_id']])
            oc_client.kv_store('DELETE', key=assoc_id)
            raise bgpvpn_ext.BGPVPNNetAssocNotFound(id=assoc_id,
                                                    bgpvpn_id=bgpvpn_id)

        net_assoc = utils.make_net_assoc_dict(net_assoc['id'],
                                              net_assoc['tenant_id'],
                                              net_assoc['bgpvpn_id'],
                                              net_assoc['network_id'],
                                              fields)
        return net_assoc

    def get_net_assocs(self, context, bgpvpn_id, filters=None, fields=None):
        LOG.debug("get_net_assocs called for bgpvpn %s, fields = %s, "
                  "filters = %s" % (bgpvpn_id, fields, filters))

        oc_client = self._get_opencontrail_api_client(context)

        get_fields = ['tenant_id', 'route_targets', 'import_targets',
                      'export_targets']
        bgpvpn = self.get_bgpvpn(context, bgpvpn_id, fields=get_fields)

        bgpvpn_net_assocs = []
        for kv_dict in oc_client.kv_store('RETRIEVE'):
            try:
                value = jsonutils.loads(kv_dict['value'])
            except ValueError:
                continue
            if (isinstance(value, dict) and
                    'bgpvpn_net_assoc' in value and
                    utils.filter_resource(value['bgpvpn_net_assoc'],
                                          filters) and
                    value['bgpvpn_net_assoc']['bgpvpn_id'] == bgpvpn_id):
                net_assoc = value['bgpvpn_net_assoc']
                # If the network was delete all bgpvpn related association
                # should be deleted also
                try:
                    oc_client.id_to_fqname(net_assoc['network_id'])
                except oc_exc.OpenContrailAPINotFound:
                    self._set_bgpvpn_association(oc_client, 'DELETE', bgpvpn,
                                                 [net_assoc['network_id']])
                    oc_client.kv_store('DELETE', key=net_assoc['id'])
                    continue
                net_assoc = utils.make_net_assoc_dict(net_assoc['id'],
                                                      net_assoc['tenant_id'],
                                                      net_assoc['bgpvpn_id'],
                                                      net_assoc['network_id'],
                                                      fields)
                bgpvpn_net_assocs.append(net_assoc)

        return bgpvpn_net_assocs

    def delete_net_assoc(self, context, assoc_id, bgpvpn_id):
        LOG.debug("delete_net_assoc called for %s" % assoc_id)
        net_assoc = self.get_net_assoc(context, assoc_id, bgpvpn_id)
        fields = ['type', 'route_targets', 'import_targets', 'export_targets']
        bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'],
                                 fields=fields)
        oc_client = self._get_opencontrail_api_client(context)

        self._set_bgpvpn_association(oc_client, 'DELETE', bgpvpn,
                                     [net_assoc['network_id']])
        oc_client.kv_store('DELETE', key=assoc_id)

        return net_assoc

    def create_router_assoc(self, context, bgpvpn_id, router_association):
        raise bgpvpn_ext.BGPVPNRouterAssociationNotSupported(
            driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)

    def get_router_assoc(self, context, assoc_id, bgpvpn_id, fields=None):
        raise bgpvpn_ext.BGPVPNRouterAssociationNotSupported(
            driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)

    def get_router_assocs(self, context, bgpvpn_id, filters=None, fields=None):
        raise bgpvpn_ext.BGPVPNRouterAssociationNotSupported(
            driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)

    def delete_router_assoc(self, context, assoc_id, bgpvpn_id):
        raise bgpvpn_ext.BGPVPNRouterAssociationNotSupported(
            driver=OPENCONTRAIL_BGPVPN_DRIVER_NAME)



# Copyright (c) 2017 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from openstack_dashboard.test import helpers

from bgpvpn_dashboard.api import bgpvpn as bgpvpn_api
from bgpvpn_dashboard.dashboards.project.bgpvpn import forms as bgpvpn_form


class TestEditDataBgpVpn(helpers.APITestCase):

    def setUp(self):
        super(TestEditDataBgpVpn, self).setUp()
        self.mock_request = mock.MagicMock()
        self.bgpvpn_form = bgpvpn_form.EditDataBgpVpn(self.mock_request)
        self.bgpvpn_form.action = "update"

    @mock.patch.object(bgpvpn_form, 'bgpvpn_api')
    def test_clean(self, mock_bgpvpn_api):
        expected_result = {"bgpvpn_id": "foo-id",
                           "name": "foo-updated"}
        list_bgpvpns = [bgpvpn_api.Bgpvpn({"id": "foo-id", "name": "foo"})]
        mock_bgpvpn_api.bgpvpns_list.return_value = list_bgpvpns
        self.bgpvpn_form.cleaned_data = expected_result
        result = self.bgpvpn_form.clean()

        self.assertEqual(expected_result, result)

    @mock.patch.object(bgpvpn_form, 'bgpvpn_api')
    def test_handle(self, mock_bgpvpn_api):
        self.bgpvpn_form.request.user.is_superuser = False
        test_data = {"bgpvpn_id": "foo-id",
                     "name": "foo-name",
                     "type": "l3"}
        expected_data = bgpvpn_api.Bgpvpn({"id": "foo-id",
                                           "name": "foo-name",
                                           "type": "l3"})
        mock_bgpvpn_api.bgpvpn_update.return_value = expected_data
        result = self.bgpvpn_form.handle(self.mock_request, test_data)

        self.assertEqual(expected_data, result)
        mock_bgpvpn_api.bgpvpn_update.assert_called_once_with(
            self.mock_request, "foo-id", name="foo-name")



# Copyright (c) 2015 Ericsson.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_bgpvpn_tempest.services.bgpvpn import bgpvpn_client
import tempest.api.network.base as test
from tempest import config
from tempest.lib.common.utils import data_utils

CONF = config.CONF


class BaseBgpvpnTest(test.BaseNetworkTest):
    """Base class for the Bgpvpn tests that use the Tempest Neutron REST client

    Finally, it is assumed that the following option is defined in the
    [service_available] section of etc/tempest.conf

        bgpvpn as True

    """

    credentials = ['primary', 'admin', 'alt']
    bgpvpn_client = None
    bgpvpn_admin_client = None
    bgpvpn_alt_client = None

    @classmethod
    def resource_cleanup(cls):
        for bgpvpn in cls.bgpvpns:
            cls.bgpvpn_admin_client.delete_bgpvpn(bgpvpn['id'])
        super(BaseBgpvpnTest, cls).resource_cleanup()

    @classmethod
    def resource_setup(cls):
        cls.route_distinguishers = []
        cls.bgpvpns = []
        cls.bgpvpn_client = bgpvpn_client.BgpvpnClient(
            cls.os.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **cls.os.default_params)
        cls.bgpvpn_admin_client = bgpvpn_client.BgpvpnClient(
            cls.os_adm.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **cls.os_adm.default_params)
        cls.bgpvpn_alt_client = bgpvpn_client.BgpvpnClient(
            cls.alt_manager.auth_provider,
            CONF.network.catalog_type,
            CONF.network.region or CONF.identity.region,
            endpoint_type=CONF.network.endpoint_type,
            build_interval=CONF.network.build_interval,
            build_timeout=CONF.network.build_timeout,
            **cls.alt_manager.default_params)
        super(BaseBgpvpnTest, cls).resource_setup()

    @classmethod
    def skip_checks(cls):
        super(BaseBgpvpnTest, cls).skip_checks()
        if not CONF.service_available.bgpvpn:
            raise cls.skipException("Bgpvpn support is required")

    def create_bgpvpn(self, client, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name('test-bgpvpn-')

        body = client.create_bgpvpn(**kwargs)
        bgpvpn = body['bgpvpn']
        self.bgpvpns.append(bgpvpn)
        return bgpvpn



# Copyright (c) 2017 Juniper Networks, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron.tests import base

from networking_bgpvpn.neutron.services.common.utils import filter_resource


class TestFilterResource(base.BaseTestCase):
    _fake_resource_string = {
        'fake_attribute': 'fake_value1',
    }
    _fake_resource_list = {
        'fake_attribute': ['fake_value1', 'fake_value2', 'fake_value3'],
    }

    def test_filter_resource_succeeds_with_one_value(self):
        filters = {
            'fake_attribute': 'fake_value1',
        }
        self.assertTrue(filter_resource(self._fake_resource_string, filters))
        self.assertTrue(filter_resource(self._fake_resource_list, filters))

    def test_filter_resource_fails_with_one_value(self):
        filters = {
            'fake_attribute': 'wrong_fake_value1',
        }
        self.assertFalse(filter_resource(self._fake_resource_string, filters))
        self.assertFalse(filter_resource(self._fake_resource_list, filters))

    def test_filter_resource_succeeds_with_list_of_values(self):
        filters = {
            'fake_attribute': ['fake_value1'],
        }
        self.assertTrue(filter_resource(self._fake_resource_string, filters))
        filters = {
            'fake_attribute': ['fake_value1', 'fake_value2'],
        }
        self.assertTrue(filter_resource(self._fake_resource_list, filters))

    def test_filter_resource_fails_with_list_of_values(self):
        filters = {
            'fake_attribute': ['wrong_fake_value1'],
        }
        self.assertFalse(filter_resource(self._fake_resource_string, filters))
        filters = {
            'fake_attribute': ['wrong_fake_value1', 'fake_value2'],
        }
        self.assertFalse(filter_resource(self._fake_resource_list, filters))



# Copyright (c) 2016 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutronclient.v2_0 import client
import os
import sys

# Parameter for subnet neutron object
SUBNET_IP = "192.168.24.0/24"

# Parameters for bgpvpn neutron object
BGPVPN_RT = "64512:2"


# Function to obtain stack parameters from system vars
def get_keystone_creds():
    d = {}
    try:
        d['username'] = os.environ['OS_USERNAME']
        d['password'] = os.environ['OS_PASSWORD']
        d['auth_url'] = os.environ['OS_AUTH_URL']
        d['tenant_name'] = os.environ['OS_TENANT_NAME']
    except KeyError:
        print("ERROR: Stack environment variables type "
              "OS_* are not properly set")
        sys.exit(1)
    return d


# Main function
def main():
    # Call function that imports (dev)stack vars
    creds = get_keystone_creds()

    # Neutron object
    # It dynamically loads the BGPVPN API
    neutron = client.Client(**creds)

    try:
        # Network object creation. This dummy network will be used to bind the
        # attached subnet to the BGPVPN object.

        # Creation of the Network
        net_obj = neutron.create_network({'network': {'name': "dummyNet"}})
        # Verify creation
        print('Network created\t[network-id:%s]...' %
              net_obj['network']['id'])

        # Creation of the subnet, is attached to the created network
        subnet_obj = neutron.create_subnet(
            {'subnet':
                {'name': "dummySubnet",
                 'cidr': SUBNET_IP,
                 'network_id': net_obj['network']['id'],
                 'ip_version': 4}})
        # Verify
        print("Subnet created\t[subnet-id:%s]..." %
              subnet_obj['subnet']['id'])

        # Creation of a BGPVPN object. This object is created with the
        # required parameter 'routes_targets'.
        # This object can be created with others parameters or be updated with
        # them by calling the update function on the object.

        print("\nBGPVPN object handling.")
        # Creation of the BGPVPN object
        bgpvpn_obj = neutron.create_bgpvpn(
            {'bgpvpn': {'route_targets': [BGPVPN_RT]}})
        print("BGPVPN object created\t[bgpvpn-id:%s]..." %
              bgpvpn_obj['bgpvpn']['id'])
        # Update the BGPVPN object
        bgpvpn_obj = neutron.update_bgpvpn(
            bgpvpn_obj['bgpvpn']['id'], {'bgpvpn': {'name': "dummyBGPVPN"}})
        # List all BGPVPN objects
        list_bgpvpn_obj = neutron.list_bgpvpns()
        print("List of all BGPVPN object\t[%s]" % list_bgpvpn_obj)
        # List of all BGPVPN objects filtered on the type parameter set to l3
        # value
        list_bgpvpn_obj = neutron.list_bgpvpns(type='l3')
        print("List of all BGPVPN object with type=l3\t[%s]" %
              list_bgpvpn_obj)

        # Creation of a BGPVPN Network association.
        print("\nBGPVPN Network Association object handling.")
        # Creation of a Network Association bound on the created BGPVPN object
        bgpvpn_net_assoc_obj = neutron.create_network_association(
            bgpvpn_obj['bgpvpn']['id'],
            {'network_association':
                {'network_id':
                    net_obj['network']['id']}})
        print("BGPVPN Network Association created\t"
              "[network_association:%s]..." %
              bgpvpn_net_assoc_obj['network_association']['id'])
        # List all NETWORK ASSOCIATION object filtered on the network created
        # above
        list_bgpvpn_net_assoc_obj = neutron.list_network_associations(
            bgpvpn_obj['bgpvpn']['id'],
            network_id=net_obj['network']['id'])
        print("List of NETWORK ASSOCIATION objects using network_id"
              "[%s]\t[%s]" %
              (net_obj['network']['id'], list_bgpvpn_net_assoc_obj))

        # Deletion of all objects created in this example

        print("\nDeletion of all created objects")
        # First declared associations related of the created BGPVPN object in
        # this example
        neutron.delete_network_association(
            bgpvpn_net_assoc_obj['network_association']['id'],
            bgpvpn_obj['bgpvpn']['id'])
        # Then the BGPVPN object
        neutron.delete_bgpvpn(bgpvpn_obj['bgpvpn']['id'])
        # Subnet
        neutron.delete_subnet(subnet_obj['subnet']['id'])
        # And finally the Network
        neutron.delete_network(net_obj['network']['id'])
    except Exception as e:
        print("[ERROR][%s]" % str(e))
        sys.exit(1)

    print("[Done]")

if __name__ == '__main__':
        main()

__all__ = ['main']



# Copyright (c) 2015 Orange.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

import six

from neutron.api import extensions
from neutron.api.v2 import base
from neutron.api.v2 import resource_helper

from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
from neutron_lib.api import extensions as api_extensions
from neutron_lib import exceptions as n_exc
from neutron_lib.plugins import directory
from neutron_lib.services import base as libbase

from oslo_log import log

from networking_bgpvpn._i18n import _

from networking_bgpvpn.neutron import extensions as bgpvpn_ext

LOG = log.getLogger(__name__)


extensions.append_api_extensions_path(bgpvpn_ext.__path__)


class BGPVPNNotFound(n_exc.NotFound):
    message = _("BGPVPN %(id)s could not be found")


class BGPVPNNetAssocNotFound(n_exc.NotFound):
    message = _("BGPVPN network association %(id)s could not be found "
                "for BGPVPN %(bgpvpn_id)s")


class BGPVPNRouterAssocNotFound(n_exc.NotFound):
    message = _("BGPVPN router association %(id)s could not be found "
                "for BGPVPN %(bgpvpn_id)s")


class BGPVPNTypeNotSupported(n_exc.BadRequest):
    message = _("BGPVPN %(driver)s driver does not support %(type)s type")


class BGPVPNRDNotSupported(n_exc.BadRequest):
    message = _("BGPVPN %(driver)s driver does not support to manually set "
                "route distinguisher")


class BGPVPNFindFromNetNotSupported(n_exc.BadRequest):
    message = _("BGPVPN %(driver)s driver does not support to fetch BGPVPNs "
                "associated to network id %(net_id)")


class BGPVPNNetAssocAlreadyExists(n_exc.BadRequest):
    message = _("network %(net_id)s is already associated to "
                "BGPVPN %(bgpvpn_id)s")


class BGPVPNRouterAssociationNotSupported(n_exc.BadRequest):
    message = _("BGPVPN %(driver)s driver does not support router "
                "associations")


class BGPVPNRouterAssocAlreadyExists(n_exc.BadRequest):
    message = _("router %(router_id)s is already associated to "
                "BGPVPN %(bgpvpn_id)s")


class BGPVPNMultipleRouterAssocNotSupported(n_exc.BadRequest):
    message = _("BGPVPN %(driver)s driver does not support multiple "
                "router association with a bgpvpn")


class BGPVPNNetworkAssocExistsAnotherBgpvpn(n_exc.BadRequest):
    message = _("Network %(network)s already associated with %(bgpvpn)s. "
                "BGPVPN %(driver)s driver does not support same network"
                " associated to multiple bgpvpns")


class BGPVPNDriverError(n_exc.NeutronException):
    message = _("%(method)s failed.")


class Bgpvpn(api_extensions.ExtensionDescriptor):

    @classmethod
    def get_name(cls):
        return bgpvpn_def.NAME

    @classmethod
    def get_alias(cls):
        return bgpvpn_def.ALIAS

    @classmethod
    def get_description(cls):
        return bgpvpn_def.DESCRIPTION

    @classmethod
    def get_updated(cls):
        return bgpvpn_def.UPDATED_TIMESTAMP

    @classmethod
    def get_resources(cls):
        plural_mappings = resource_helper.build_plural_mappings(
            {}, bgpvpn_def.RESOURCE_ATTRIBUTE_MAP)
        resources = resource_helper.build_resource_info(
            plural_mappings,
            bgpvpn_def.RESOURCE_ATTRIBUTE_MAP,
            bgpvpn_def.LABEL,
            register_quota=True,
            translate_name=True)
        plugin = directory.get_plugin(bgpvpn_def.LABEL)
        for collection_name in bgpvpn_def.SUB_RESOURCE_ATTRIBUTE_MAP:
            # Special handling needed for sub-resources with 'y' ending
            # (e.g. proxies -> proxy)
            resource_name = collection_name[:-1]
            parent = bgpvpn_def.SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].\
                get('parent')
            params = bgpvpn_def.SUB_RESOURCE_ATTRIBUTE_MAP[collection_name].\
                get('parameters')

            controller = base.create_resource(collection_name, resource_name,
                                              plugin, params,
                                              allow_bulk=True,
                                              parent=parent,
                                              allow_pagination=True,
                                              allow_sorting=True)

            resource = extensions.ResourceExtension(
                collection_name,
                controller, parent,
                path_prefix='bgpvpn',
                attr_map=params)
            resources.append(resource)
        return resources

    @classmethod
    def get_plugin_interface(cls):
        return BGPVPNPluginBase

    def update_attributes_map(self, attributes):
        super(Bgpvpn, self).update_attributes_map(
            attributes, extension_attrs_map=bgpvpn_def.RESOURCE_ATTRIBUTE_MAP)

    def get_extended_resources(self, version):
        if version == "2.0":
            return bgpvpn_def.RESOURCE_ATTRIBUTE_MAP
        else:
            return {}


@six.add_metaclass(abc.ABCMeta)
class BGPVPNPluginBase(libbase.ServicePluginBase):

    def get_plugin_name(self):
        return bgpvpn_def.LABEL

    def get_plugin_type(self):
        return bgpvpn_def.LABEL

    def get_plugin_description(self):
        return 'BGP VPN Interconnection service plugin'

    @abc.abstractmethod
    def create_bgpvpn(self, context, bgpvpn):
        pass

    @abc.abstractmethod
    def get_bgpvpns(self, context, filters=None, fields=None):
        pass

    @abc.abstractmethod
    def get_bgpvpn(self, context, id, fields=None):
        pass

    @abc.abstractmethod
    def update_bgpvpn(self, context, id, bgpvpn):
        pass

    @abc.abstractmethod
    def delete_bgpvpn(self, context, id):
        pass

    @abc.abstractmethod
    def create_bgpvpn_network_association(self, context, bgpvpn_id,
                                          network_association):
        pass

    @abc.abstractmethod
    def get_bgpvpn_network_association(self, context, assoc_id, bgpvpn_id,
                                       fields=None):
        pass

    @abc.abstractmethod
    def get_bgpvpn_network_associations(self, context, bgpvpn_id,
                                        filters=None, fields=None):
        pass

    @abc.abstractmethod
    def update_bgpvpn_network_association(self, context, assoc_id, bgpvpn_id,
                                          network_association):
        pass

    @abc.abstractmethod
    def delete_bgpvpn_network_association(self, context, assoc_id, bgpvpn_id):
        pass

    @abc.abstractmethod
    def create_bgpvpn_router_association(self, context, bgpvpn_id,
                                         router_association):
        pass

    @abc.abstractmethod
    def get_bgpvpn_router_association(self, context, assoc_id, bgpvpn_id,
                                      fields=None):
        pass

    @abc.abstractmethod
    def get_bgpvpn_router_associations(self, context, bgpvpn_id, filters=None,
                                       fields=None):
        pass

    @abc.abstractmethod
    def delete_bgpvpn_router_association(self, context, assoc_id, bgpvpn_id):
        pass
